/*
BigClock

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Written by Jens Rupp
jens@gacel.de
http://www.gacel.de/bigclock.htm
*/
#define NON_PORTABLE

#include <PalmOS.h>
#include <CoreTraps.h>
#include <System/SerialMgrOld.h>   // for talelight
#include <SysEvtMgr.h>
#include <PalmCompatibility.h>
#include "callback.h"
#include "bigclock.h"

#define BCAppID       'BClk'
#define ytop 15         // upper panel
#define ybottom 77      // lower panel
#define ymove 2

//not in include files




//default panels
const int defpages[2][8] ={
                       {pnMonth << 3 ,pnTime << 3, (pnAlarm << 3) +0,(pnAlarm << 3) +1,(pnTime << 3) + 1,(pnTime << 3) + 2,pnTimer << 3,pnControl << 3},
                       {pnDate << 3,(pnTime << 3) , (pnAlarm << 3) +2,(pnAlarm << 3) +3,pnWorld << 3,(pnWorld << 3) +1, (pnTimer << 3) + 1,(pnControl << 3) + 1}
                      };


// default sounds
const unsigned int inits[4][4][3] = {
                        { {1000,200,0} , {2000,200,0} , {0,0,0}, {0,0,0}},
                        { {500,100,0} , {1000,100,0} , {500,100,0}, {1000,100,50}},
                        { {1000,100,0} , {2000,100,0} , {4000,100,0}, {5000,100,300}},
                        { {2000,300,0} , {4000,300,0} , {2000,200,0}, {4000,200,20}}
                       }; //96

//for analog clock
//const int yv[61] = {-512,-509,-500,-486,-467,-443,-414,-380,-342,-300,-256,-208,-158,-106,-53,0,53,106,158,208,255,300,342,380,414,443,467,486,500,509,512,509,500,486,467,443,414,380,342,300,256,208,158,106,53,0,-53,-106,-158,-208,-256,-300,-342,-380,-414,-443,-467,-486,-500,-509,-512};
//const int xv[61] = {0,53,106,158,208,256,300,342,380,414,443,467,486,500,509,512,509,500,486,467,443,414,380,342,300,256,208,158,106,53,0,-53,-106,-158,-208,-255,-300,-342,-380,-414,-443,-467,-486,-500,-509,-512,-509,-500,-486,-467,-443,-414,-380,-342,-300,-256,-208,-158,-106,-53,0};


// sound definition
typedef struct
         {
            Boolean   beep;
            int  freq[4];
            int  time[4];
            int  amplitude[4];
            int  pause[4];
            Char  count;
         } BigClockSound; //34 byte

typedef struct
        {
          Word           active     :1 ;    // is alarm active
          Word           ampm       :1 ;    // display ampm
          Word           seconds    :1 ;    // display seconds
          Word           backlight  :1 ;    // activate backlight
          Word           wait       :1 ;    // autoclose alarm screen
          Word           timerup    :1 ;    // count timer up
          Word           sound      :2 ;    // sound to use
          Word           oldmode    :2 ;    // 0 simple
                                            // 1 reset
                                            // 2 loop
                                            // 3 alarm
          Word           snooze     :1 ;    // use snooze
          Word           base       :3 ;    // worldtime to use
          Word           trigger    :2 ;    // display day selection
          Word           silent     :1 ;    // no sound
          Word           talelight  :1 ;    // use talelight
          Word           closed     :1 ;    // manually closed
          Word           autooff    :1 ;    // keep device on
          Word           mode       :4 ;    // 0 simple
                                            // 1 reset
                                            // 2 loop
                                            // 3 alarm
                                            // 4 alarm&loop
                                            // 5 go on
                                            // 6 alarm and go on
          Word           daychanged :1 ;    // flag set if day or month or year is changed since last redraw
          Word           daterelative:1;    // date relative timer
          Word           alternate :1;      // chessmode
          Word           ignoresystemsoundoff:1; //required to store in alarm
          Word           calc:1;             //calculated timer
        } BigClockOptions;    //4


typedef struct
        {
          Char set;
          Char page;
          Char start;
          Char end;
          UInt16 id;
        } BackgroundInfo;


typedef struct
        {
          Word numbers;
          Word count;
          BackgroundInfo back;
        } BigClockThemeDef;


typedef struct
        {
          DateTimeType   time;          // time of alarm
          BigClockOptions  options;
          BigClockSound  sound;         // what alarm to sound
          unsigned char          days;          // days to sound alarm
          char           name[12];      // name of alarm
          char           worldname[12]; // name of the worldtime used
          long           worldoffset;   // offset of the worldtime used
          Word           snoozetime;    // time between snooze alarms
          Char           snoozecount;   // number of snooze alarms
          Char           snoozecurrent; // number of snooze to sound
          unsigned long  snoozeoffset;  // number of seconds to add for snooze;
              } BigClockAlarm; //80

typedef struct
        {
          char             name[12];      // name of time
          long             value;         // current value
          long             reset;         // reset value
          long             date;          // date for relative timer
          BigClockOptions  options;        // options
          ULong            last;          // last displayed value
          Word             snoozetime;    // snooze time in minutes
          Char             snoozecount;   // number of snooze
          unsigned char            days;          // days to sound alarm
          Int8             add;           // display x add
          int              monthoffset;   // month offset for month view
          int              weekcalc;      // offset for weeknumbr calculation
          long             calc;
        } BigClockTime; //40


typedef struct
	{
          Word    version;
          BigClockTime     world[wCount];
          BigClockSound    sounds[4];
          Char mode;
          Char pageset;
          unsigned char pages[2][8];
          char pagenames[2][4][9];
          IndexedColorType colorb;
          IndexedColorType colorf;
          IndexedColorType colorbi;
          IndexedColorType colorfi;
          char history[8][12];
          char theme[16];
          Boolean  hardwarebuttons; // use up/down for timer
          Boolean  ignoresystemsoundoff;
	} BigClockPrefType;


static int StartApplication(void);
static Boolean ReadAlarm(DmOpenRef BCdb,int num,BigClockAlarm *target);
static Boolean WriteAlarm(DmOpenRef BCdb,int num,BigClockAlarm *source);
static Boolean bigclock(EventPtr event);
static Boolean layout(EventPtr event);
static Boolean optionsworld(EventPtr event);
static Boolean optionsglobal(EventPtr event);
static Boolean optionssound(EventPtr event);
static Boolean optionscolor(EventPtr event);
static Boolean optionslocaltime(EventPtr event);
static Boolean optionsalarm(EventPtr event);
static Boolean labelhistory(EventPtr event);
static Boolean newyear(EventPtr event);
static void EventLoop(void);
static void StopApplication(void);
void SetNextAlarm(UInt cno,LocalID aid);
void CloseAlarm(DmOpenRef BCdb);
DmOpenRef OpenAlarm();
void SwitchAlarm();
void AlarmTriggered (SysAlarmTriggeredParamType * cmdPBP);
void DisplayAlarm (SysAlarmTriggeredParamType * cmdPBP);
void SetSound(int a,int w);
void WorldTimeDraw();
void ResetData();
//static void SetFieldText(int id_in, char *text_in_t, UInt length);
void UpdateSystemAlarm ();
void UpdateTimer(int num);
void DrawAnalogTime(int num,int y);
//Boolean	HwrBacklight(Boolean set, Boolean newState);

Boolean	HwrBacklight(Boolean set, Boolean newState) SYS_TRAP(sysTrapHwrBacklightV33);

//#include "util.c"

char help[256];       // used to store different data ( Theme list) and debugging
Boolean dirty=false;  // for the backup bit of the data database
DWord romversion;
Boolean is35;         // os 3.5 or higher
Boolean iscolor;      // color device
int   changed[2];     // upper (0) or lower (1) panel changed -> redraw

// for new year fireworks
extern int newyeardone;
/*
int maxrockets=60;
rocket *rocketlist;
VoidHand  rocketimageh[6];
BitmapPtr rocketimagep[6];
*/
//20

VoidHand hdays;
char      *pdays;
WinHandle bitmaps;
WinHandle build;
DateTimeType Time[wCount];

VoidHand   bigbitmapH[16];
BitmapPtr  bigbitmapP[16];

DmOpenRef  ThemeDB=0;
int        BackgroundID=0;
VoidHand   BackgroundH;
BitmapPtr  Background=NULL;
VoidHand   ThemeDefH;
BigClockThemeDef *ThemeDef=NULL;
char       *ThemeList[maxthemes];


//61

SystemPreferencesType SysPref; //100
DateTimeType LastTime[1];
DmOpenRef     SysDB;
//108

//day and month text parsed from resource
char      daystrings[100];
char      monthstrings[100];
char      *day[7];
char      *month[12];

char      page=0;
char      optionpage=0;
char      historymode;
char      detailweek[12];
char      detailday[12];
char      detailhour[12];

char      stimer[4][12];

char      salarmmode[2][12];
char      stimermode[7][14];

char      reltimerdate[12];
char      reltimertime[12];
//484

static UInt cardNo;
LocalID appID;
//8

// midi support...  still not done :(
//Boolean   usemidi;
//char   **midilist=NULL;

BigClockPrefType prefs;

Boolean ao;
int     obase;
int     timercommand=0;
//6

const inot[2] = {1,0};


DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  int error;

  if (cmd == sysAppLaunchCmdNormalLaunch)
  {
    error = StartApplication();	// Application start code

    if (error) return error;
    EventLoop();	// Event loop

    StopApplication ();	// Application stop code
  } else if (cmd == sysAppLaunchCmdAlarmTriggered)
  {
    AlarmTriggered ((SysAlarmTriggeredParamType *)cmdPBP);
  } else if (cmd == sysAppLaunchCmdDisplayAlarm)
  {
    DisplayAlarm ((SysAlarmTriggeredParamType *)cmdPBP);
  } else if ((cmd == sysAppLaunchCmdSystemReset) ||
             (cmd == sysAppLaunchCmdTimeChange)
             )

  {
    UpdateSystemAlarm();
  }
  return (0);
}

int DisplayAndSound(SysAlarmTriggeredParamType * cmdPBP,int typ)
{
  FormPtr frm;
  FormPtr curForm;
  EventType	event;
  RectangleType rc1 = {{0,0},{160,40}};
  RectangleType rc2 = {{0,40},{160,120}};
  RectangleType rc3 = {{16,20},{128,18}};
  int dummy;
  DateTimeType time;
  BigClockAlarm *alarm;
  SystemPreferencesType *lSysPref;
  DWord backlight;
  int note;
  int rep;
  Boolean quit;
  SndCommandType cmd;
  int stop;
  DmOpenRef BCdb;
  Boolean del;
  Boolean blok;
  UInt    serialRef = -1;
  int     result=0;
  char    message[32];
  char    number[6];
  WinHandle Save;
  FontID oldfont;
  RectangleType screen = {{0,0},{160,160}};
  DWord version;
  int err;


  // create offscreen window to save screen
  Save=WinCreateOffscreenWindow(160,160,screenFormat,(Word *)&dummy);
  // save screen
  if (Save!=NULL)
    WinCopyRectangle(WinGetDisplayWindow(),Save,&screen,0,0,scrCopy);

  //remeber old form
  curForm = FrmGetActiveForm ();
  //remember old font
  oldfont = FntGetFont();


  //set new form
  frm= FrmGetFormPtr(formID_displayalarm);
  if (frm==NULL)
    {
      // if not created already, create it
      frm = FrmInitForm (formID_displayalarm);
      del=true;
    }
  else
    del=false;  // remember not to delete the form
  FrmSetActiveForm (frm);



  //display form
  FntSetFont(2);
  //bug, depending on compile this is required or creates a exception

  err = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &version);
  //StrIToA(message,version);
  //FrmCustomAlert(alertID_text,message,NULL,NULL);

  if(version < 0x03500000)
    WinEraseWindow();
  FrmDrawForm (frm);


  // read preferences
  lSysPref=MemPtrNew(sizeof(SystemPreferencesType));
  if (lSysPref==NULL) return result;
  PrefGetPreferences(lSysPref);

  //read alarm data
  alarm=MemPtrNew(sizeof(BigClockAlarm));
  if (alarm==NULL) return result;
  BCdb=OpenAlarm();
  ReadAlarm(BCdb,cmdPBP->ref,alarm);
  CloseAlarm(BCdb);

  // display data
  TimSecondsToDateTime(cmdPBP->alarmSeconds,&time);
  TimAdjust(&time,alarm->worldoffset);
  TimeToAscii(time.hour,time.minute,lSysPref->timeFormat,message);
  WinDrawChars(alarm->worldname,StrLen(alarm->worldname),2,90);
  WinDrawChars(message,StrLen(message),2,106);
  WinDrawChars(alarm->name,StrLen(alarm->name),2,122);

  //display snooze
  if (alarm->options.snooze==1)
  {
    SysCopyStringResource(message,stringID_stopsnooze);
    FntSetFont(0);
    dummy=FntCharsWidth(message,StrLen(message));
    WinDrawRectangleFrame(roundFrame,&rc3);
    WinDrawChars(message,StrLen(message),80-(dummy /2),24);
    if (alarm->snoozecurrent > 0)
    {
       StrCopy(message,"Snooze ");
       StrIToA(number,alarm->snoozecurrent);
       StrCat(message,number);
       StrCat(message," / ");
       StrIToA(number,alarm->snoozecount);
       StrCat(message,number);
       FntSetFont(2);
       WinDrawChars(message,StrLen(message),2,138);
    }
  }

  // use backlight?
  blok=false;
  if  ((alarm->options.backlight > 0) && (typ==1))
  {
    FtrGet(sysFtrCreator, sysFtrNumBacklight, &backlight);
    blok=(backlight>0);
  }

  // use talelight?
  if ((alarm->options.talelight >0) && (typ==1))
  {
    dummy=SysLibFind("Serial Library", &serialRef);
    if (dummy) serialRef=-1;
  }

  //clear events
  do
  {
    EvtGetEvent(&event, 1);
    if (SysHandleEvent (&event)) continue;
  } while (event.eType != nilEvent);

  note=0;
  rep=0;
  dummy=1;
  quit=false;

  if ((alarm->options.wait==0) || (typ==1))
    stop=1;
  else
    stop=-1;

  do
    {
     EvtGetEvent(&event, dummy);

    if (SysHandleEvent (&event)) continue;

    if (event.eType == penDownEvent)
    {
      if (event.screenY < 40)
      {
        WinInvertRectangle(&rc1,0);
        result=1;
      } else
      {
        WinInvertRectangle(&rc2,0);
      }
    }


    if (!quit)
      {
        if (typ==1)  // play sound
        {
          if (rep < alarm->sound.count) // number of repetition?
            {
              //backlight
              if (blok) HwrBacklight(true,true);
              // talelight
              if (serialRef != -1) SerOpen(serialRef, 0, 19200L);

              //Sound alarm?
              if (
                   (alarm->options.silent == 0) &&
                   (
                     (lSysPref->alarmSoundLevelV20 != 1) ||
                     (alarm->options.ignoresystemsoundoff)
                   )
                 )  
              {
                // system sound?
                if (alarm->sound.beep)
                {
                  //play a system alarm
                  SndPlaySystemSound(sndAlarm);
                } else
                {
                  if (alarm->sound.freq[note] > 0)
                  {
                    //play the note
                    cmd.cmd = sndCmdFreqDurationAmp;
                    cmd.param1 = alarm->sound.freq[note];
                    cmd.param2 = alarm->sound.time[note];
                    cmd.param3 = alarm->sound.amplitude[note];
                    SndDoCmd(0, &cmd, 0);
                  }
                }

              }
              //talelight off
              if (serialRef != -1) SerClose(serialRef);
              //backlight off
              if (blok) HwrBacklight(true,false);

              //set pause
              dummy=alarm->sound.pause[note];

              note++;  //next note
              if ((note==4) || (alarm->sound.beep)) // last note in set of 4?
              {
                rep++;  // next repetition
                note=0; // first note
              }
            }
          else
            dummy=stop;
        }
         else
           {
             dummy=stop;
             rep=alarm->sound.count+1;
           }
      }

  //check key
  if (event.eType == keyDownEvent)
    if (
       (event.data.keyDown.chr == pageDownChr) ||
       (event.data.keyDown.chr == pageUpChr)
       )
     {
       quit=true;
     }
  //check pen
  if (event.eType == penUpEvent)
    quit=true;


 } while (
          (! quit) &&
          (! (
               (
                 (alarm->options.wait==0) ||
                 (typ==1)
                ) &&
               (rep >= alarm->sound.count)
             )
          ) /*&&
          (event.eType != appStopEvent)*/
         );

  // close serial
  if (serialRef != -1) SerClose(serialRef);
  // keep backlight on
  if (blok) HwrBacklight(true,true);

  if (quit)
    alarm->options.closed=1;
  else
    alarm->options.closed=0;

  // remember closed alarm
  BCdb=OpenAlarm();
  WriteAlarm(BCdb,cmdPBP->ref,alarm);
  CloseAlarm(BCdb);

  //free memory
  MemPtrFree(lSysPref);
  MemPtrFree(alarm);

  // restore old font
  FntSetFont(oldfont);

  //delete form
  //FrmCustomAlert(alertID_text,"del",NULL,NULL);

  if (del)
  {
    //FrmEraseForm(frm);  bug, causes fatal error with launcherIII, i restore the screen later anyway
    FrmDeleteForm (frm);
  }
  //FrmCustomAlert(alertID_text,"form",NULL,NULL);

  //restore active form
  FrmSetActiveForm (curForm);

  // restore screen
  //FrmCustomAlert(alertID_text,"restore",NULL,NULL);
  if (Save != NULL)
  {
    WinCopyRectangle(Save,WinGetDisplayWindow(),&screen,0,0,scrCopy);
    WinDeleteWindow(Save,false);
  }
  //FrmCustomAlert(alertID_text,"return",NULL,NULL);
  return result;
}

void UpdateSystemAlarm ()
{
  UInt cardNo;
  LocalID dbID;
  DmSearchStateType searchInfo;

  DmGetNextDatabaseByTypeCreator (true, &searchInfo,
     sysFileTApplication, BCAppID, true, &cardNo, &dbID);

  SetNextAlarm(cardNo,dbID);
}



void AlarmTriggered (SysAlarmTriggeredParamType * cmdPBP)
{
  //UInt cardNo;
  //LocalID dbID;
  //DmSearchStateType *searchInfo;
  int r;
  BigClockAlarm *alarm;
  DmOpenRef BCdb;



  UpdateSystemAlarm();
  //find application database
  /*
  searchInfo=MemPtrNew(sizeof(DmSearchStateType));
  DmGetNextDatabaseByTypeCreator (true, searchInfo,
     sysFileTApplication, BCAppID, true, &cardNo, &dbID);
  MemPtrFree(searchInfo);

  SetNextAlarm(cardNo,dbID);
 */

  r=0;
  if (cmdPBP->ref != -2)
    r=DisplayAndSound(cmdPBP,1);

  if ((r!=0) && (cmdPBP->ref >= 0))
  {
     alarm=MemPtrNew(sizeof(BigClockAlarm));
     BCdb=OpenAlarm();
     ReadAlarm(BCdb,cmdPBP->ref,alarm);
     if (alarm->options.snooze == 1)
     {
       // set to end of snooze
       alarm->snoozecurrent=alarm->snoozecount+1;
       WriteAlarm(BCdb,cmdPBP->ref,alarm);
     }
     CloseAlarm(BCdb);
     MemPtrFree(alarm);
  }
}


void DisplayAlarm (SysAlarmTriggeredParamType * cmdPBP)
{
  UInt cardNo;
  LocalID dbID;
  DmSearchStateType searchInfo;
  DateTimeType time;
  BigClockAlarm *alarm;
  //long snoozetime;
  unsigned long now;
  unsigned long base;
  //DWord backlight;
  DmOpenRef BCdb;
  int r=0;
  char lhelp[10];
  //char  number[8];
  //EventType event;


  if (cmdPBP->ref == -2) return;


  DmGetNextDatabaseByTypeCreator (true, &searchInfo,
     sysFileTApplication, BCAppID, true, &cardNo, &dbID);

  alarm=MemPtrNew(sizeof(BigClockAlarm));

  BCdb=OpenAlarm();
  ReadAlarm(BCdb,cmdPBP->ref,alarm);
  CloseAlarm(BCdb);

  if ((alarm->options.closed==0) &&
      (alarm->options.wait==1))
    r=DisplayAndSound(cmdPBP,0);

  //snooze
  if ((alarm->options.snooze == 1) && (r != 1))
    {
      now=TimGetSeconds();
      alarm->snoozeoffset=now+alarm->snoozetime*60;
      alarm->snoozecurrent++;

      if (alarm->snoozecurrent>alarm->snoozecount)
        {
           alarm->snoozecurrent=0;
           if (alarm->options.trigger==2)
              alarm->options.active=0;
        }

      BCdb=OpenAlarm();
      WriteAlarm(BCdb,cmdPBP->ref,alarm);
      CloseAlarm(BCdb);


      SetNextAlarm(cardNo,dbID);

    }
  else if (alarm->options.trigger==2)
    {
      alarm->options.active=0;
      BCdb=OpenAlarm();
      WriteAlarm(BCdb,cmdPBP->ref,alarm);
      CloseAlarm(BCdb);
      SetNextAlarm(cardNo,dbID);
    }
  else
    {
      alarm->snoozecurrent=0;
      BCdb=OpenAlarm();
      WriteAlarm(BCdb,cmdPBP->ref,alarm);
      CloseAlarm(BCdb);

      SetNextAlarm(cardNo,dbID);
    }

  MemPtrFree(alarm);
}

void MyDrawChars(const Char *chars, Int16 len,Coord x, Coord y)
{
  BitmapType *bitmap;
  WinHandle  win;
  WinHandle  oldwin;
  Int16 h;
  Int16 max;
  Int16 maxs;
  Boolean trunc;
  Err bmperror;

  if (len>0)
  {
    if (iscolor)
    {
      max=160;
      maxs=len;
      FntCharsInWidth (chars,&max,&maxs,&trunc);
      h=FntCharHeight();

      bitmap= BmpCreate(max, h, 8, NULL, &bmperror);
      bitmap->flags.hasTransparency=1;
      bitmap->transparentIndex=prefs.colorb;
      win = WinCreateBitmapWindow(bitmap, &bmperror);

      oldwin=WinGetDrawWindow();
      WinSetDrawWindow(win);
      WinDrawChars(chars,len,0,0);
      WinSetDrawWindow(oldwin);
      WinDeleteWindow(win,false);
      WinDrawBitmap(bitmap,x,y);
      BmpDelete(bitmap);

    } else
    {
      WinDrawChars(chars,len,x,y);
    }
  }
}


static void InitPref()
{
  int iii;
  int jjj;
  int v;
  Boolean ampm;

  MemSet(&prefs,sizeof(BigClockPrefType),0);

  prefs.version=280;

  prefs.mode=0;
  prefs.pageset=0;

  ampm=!Use24HourFormat(SysPref.timeFormat);
  if (ampm)
    v=0;
  else
    v=1;

  // 0 local 1=world1 2=world2
  for (iii=0; iii<wCount; iii++)
  {
    MemSet(&prefs.world[iii],sizeof(BigClockTime),0);
    prefs.world[iii].options.ampm=ampm;
    prefs.world[iii].options.seconds=v;
    prefs.world[iii].options.wait=1;
    prefs.world[iii].options.timerup=1;
    prefs.world[iii].options.mode=3;
    prefs.world[iii].options.trigger=1;
    prefs.world[iii].days=127;
    prefs.world[iii].snoozetime=5;
    prefs.world[iii].snoozecount=3;
    prefs.world[iii].date=TimGetSeconds();
    prefs.world[iii].calc=100;
  }
  prefs.world[wLocal].options.active=1;


  for (iii=0; iii<4; iii++)
  {
    prefs.world[wAlarm1+iii].options.seconds=false;
    prefs.world[wAlarm1+iii].value=(long)daysInSeconds*1000;
    prefs.world[wTimer1+iii].options.ampm=0;
    prefs.world[wTimer1+iii].options.seconds=1;
  }

  for (iii=0; iii<13 ; iii++)
    SysCopyStringResource(prefs.world[wLocal+iii].name,stringID_localtime+iii);

  for (iii=0; iii<2 ; iii++)
    for (jjj=0; jjj<8; jjj++)
         prefs.pages[iii][jjj]=defpages[iii][jjj];

  for (iii=0; iii<2 ; iii++)
    for (jjj=0; jjj<4 ; jjj++)
      SysCopyStringResource(prefs.pagenames[iii][jjj],stringID_pagenames00+iii*4+jjj);

  for (iii=0; iii<4; iii++)
  {
     for (jjj=0; jjj<4; jjj++)
       {
         prefs.sounds[iii].freq[jjj]=inits[iii][jjj][0];
         prefs.sounds[iii].time[jjj]=inits[iii][jjj][1];
         prefs.sounds[iii].amplitude[jjj]=sndMaxAmp;
         prefs.sounds[iii].pause[jjj]=inits[iii][jjj][2];;
       }
  }
  prefs.sounds[0].count=1;
  prefs.sounds[1].count=3;
  prefs.sounds[2].count=10;
  prefs.sounds[3].count=15;

  prefs.colorb=37;
  prefs.colorf=255;
  prefs.colorbi=255;
  prefs.colorfi=0;

}

static void DrawWorldName (Int16 itemNum, RectangleType *bounds,Char **itemsText)
{
	CharPtr	itemTextP;
	SWord		itemTextLen;
	SWord		itemWidth;

     CALLBACK_PROLOGUE

	itemTextP = prefs.world[wLocal+itemNum].name;

	itemTextLen = StrLen(itemTextP);
	// Get the width of the text
	itemWidth = FntCharsWidth(itemTextP, itemTextLen);
	// Does it fit?
	if (itemWidth <= bounds->extent.x)
		{
		// Draw entire item text as is
		MyDrawChars(itemTextP, itemTextLen, bounds->topLeft.x, bounds->topLeft.y);
		}
	else
		{
		// We're going to truncate the item text
		Boolean	ignored;
		char		ellipsisChar =0x85;
                //horizEllipsisChr;
		// Set the new max item width
		itemWidth = bounds->extent.x - FntCharWidth(ellipsisChar);
		// Find the item length that fits in the bounds minus the ellipsis
		FntCharsInWidth(itemTextP, &itemWidth, &itemTextLen, &ignored);
		// Draw item text that fits
		MyDrawChars(itemTextP, itemTextLen, bounds->topLeft.x, bounds->topLeft.y);
		// Draw ellipsis char
		MyDrawChars(&ellipsisChar, 1, bounds->topLeft.x + itemWidth, bounds->topLeft.y);
		}
  CALLBACK_EPILOGUE
}



static Boolean ReadPref()
{
  DmOpenRef BCdb;
  VoidHand  BCprefHandle;
  BigClockPrefType *BCpref=NULL;
  UInt          index = 0;
  UInt lNo;
  LocalID dbID;
  DmSearchStateType searchInfo;

  BCdb = DmOpenDatabaseByTypeCreator('data', BCAppID, dmModeReadWrite);
  // if database is does not exist create it
  if (!BCdb)
  {
    if (DmCreateDatabase(0, "BigClock data", BCAppID, 'data', false)) return 1;
    BCdb = DmOpenDatabaseByTypeCreator('data', BCAppID, dmModeReadWrite);
    BCprefHandle = DmNewRecord(BCdb, &index, sizeof(BigClockPrefType));
    BCpref = MemHandleLock(BCprefHandle);
    //init the data
    InitPref();
  } else
  {
    BCprefHandle=DmGetRecord(BCdb,0);
    BCpref=MemHandleLock(BCprefHandle);
    if (MemPtrSize(BCpref) != sizeof(BigClockPrefType))
    {
      //old version
      MemPtrUnlock(BCpref);
      DmReleaseRecord(BCdb, 0, false);
      DmResizeRecord(BCdb,0,sizeof(BigClockPrefType));
      InitPref();
      BCprefHandle=DmGetRecord(BCdb,0);
      BCpref=MemHandleLock(BCprefHandle);

      if (DmGetNextDatabaseByTypeCreator (true, &searchInfo,
         'bcal', BCAppID, true, &lNo, &dbID) == 0)
      {
        DmDeleteDatabase(lNo,dbID);
      }
      SetNextAlarm(cardNo,appID);

    } else
    MemMove(&prefs,BCpref,sizeof(BigClockPrefType));
  }
  MemPtrUnlock(BCpref);
  DmReleaseRecord(BCdb, 0, false);
  DmCloseDatabase(BCdb);
  return true;
}


static Boolean WritePref()
{
  DmOpenRef BCdb;
  VoidHand  BCprefHandle;
  BigClockPrefType *BCpref=NULL;
  //UInt          index = 0;
  UInt      attr;
  LocalID   id;

  BCdb = DmOpenDatabaseByTypeCreator('data', BCAppID, dmModeReadWrite);
  BCprefHandle=DmGetRecord(BCdb,0);
  BCpref=MemHandleLock(BCprefHandle);

  DmWrite(BCpref,0,&prefs,sizeof(BigClockPrefType));

  MemPtrUnlock(BCpref);
  DmReleaseRecord(BCdb, 0, true);
  DmCloseDatabase(BCdb);

  //for now backup always
  dirty=true;
  if (dirty)
  {
    id=DmFindDatabase(0,"BigClock data");
    if (id!=0)
    {
      DmDatabaseInfo(0,id,NULL,&attr,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
      attr=attr | dmHdrAttrBackup;
      DmSetDatabaseInfo(0,id,NULL,&attr,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
    }
  }
  return true;
}

void CloseAlarm(DmOpenRef BCdb)
{
   DmCloseDatabase(BCdb);
}


DmOpenRef OpenAlarm()
{
  VoidHand  BCalarmHandle;
  BigClockAlarm *BCalarm=NULL;
  UInt          index = 0;
  DmOpenRef BCdb;

  BCdb = DmOpenDatabaseByTypeCreator('bcal', BCAppID, dmModeReadWrite);
  // if database  does not exist create it
  if (!BCdb)
  {

    if (DmCreateDatabase(0, "Alarmlist", BCAppID, 'bcal', false)) return false;
    BCdb = DmOpenDatabaseByTypeCreator('bcal', BCAppID, dmModeReadWrite);


    for (index=0; index<aCount ; index++)
    {
      BCalarmHandle = DmNewRecord(BCdb, &index, sizeof(BigClockAlarm));
      BCalarm = MemHandleLock(BCalarmHandle);
      DmSet(BCalarm, 0, sizeof(BigClockAlarm) , 0);
      MemPtrUnlock(BCalarm);
      DmReleaseRecord(BCdb, index, false);
    }
  }
  return BCdb;
}

static Boolean ReadAlarm(DmOpenRef BCdb,int num,BigClockAlarm *target)
{
  VoidHand  BCalarmHandle;
  BigClockAlarm *BCalarm=NULL;

  BCalarmHandle = DmGetRecord(BCdb, num);

  //old version -> resize
  if (MemHandleSize(BCalarmHandle)<sizeof(BigClockAlarm))
    MemHandleResize(BCalarmHandle,sizeof(BigClockAlarm));

  BCalarm = MemHandleLock(BCalarmHandle);
  MemMove(target,BCalarm,sizeof(BigClockAlarm));

  MemPtrUnlock(BCalarm);
  DmReleaseRecord(BCdb, num, false);
  return true;
}


static Boolean WriteAlarm(DmOpenRef BCdb,int num,BigClockAlarm *source)
{
  VoidHand  BCalarmHandle;
  BigClockAlarm *BCalarm=NULL;

  BCalarmHandle = DmGetRecord(BCdb, num);
  if (MemHandleSize(BCalarmHandle)<sizeof(BigClockAlarm))
    MemHandleResize(BCalarmHandle,sizeof(BigClockAlarm));


  BCalarm = MemHandleLock(BCalarmHandle);
  DmWrite(BCalarm,0,source,sizeof(BigClockAlarm));

  MemPtrUnlock(BCalarm);
  DmReleaseRecord(BCdb, num, false);
  return true;
}


void initbigbitmap(void)
{
  VoidHand  hbmp;
  BitmapPtr pbmp;
  WinHandle oldwin;



  oldwin=WinGetDrawWindow();
  WinSetDrawWindow(bitmaps);

  hbmp=DmGetResource('Tbmp',bitmapID_large);
  pbmp=MemHandleLock(hbmp);

  if (iscolor)
  {
    WinSetForeColor(prefs.colorf);
    WinSetBackColor(prefs.colorb);
  }

  WinDrawBitmap(pbmp,0,0);
  MemHandleUnlock(hbmp);
  DmReleaseResource(hbmp);

  WinSetDrawWindow(oldwin);
}

void CopyPart(int start,int n,int w,int x,int y)
{
  int iii;
  Err bmperror;
  WinHandle  win;
  RectangleType rc = {{0,y},{w,40}};

  for (iii=0; iii<n ; iii++)
    {
      WinSetForeColor(prefs.colorf);
      WinSetBackColor(prefs.colorb);
      bigbitmapP[iii+start] = BmpCreate(w, 40, 8, NULL, &bmperror);
      if (ThemeDB != 0)
        bigbitmapP[iii+start]->flags.hasTransparency=1;
      else
        bigbitmapP[iii+start]->flags.hasTransparency=0;
      bigbitmapP[iii+start]->transparentIndex=prefs.colorb;
      win = WinCreateBitmapWindow(bigbitmapP[iii+start], &bmperror);
      rc.topLeft.x = x+w*iii;
      WinCopyRectangle(bitmaps,win,&rc,0,0,scrCopy);
      WinDeleteWindow(win,false);
    }

}

void OpenBigBitmaps(void)
{
  int iii;

  for (iii=0 ; iii<16 ; iii++)
  {
    bigbitmapH[iii]=DmGetResource('Tbmp',ThemeDef->numbers+iii);
    bigbitmapP[iii]=MemHandleLock(bigbitmapH[iii]);
  }
}

void CloseBigBitmaps(void)
{
  int iii;

  for (iii=0 ; iii<16 ; iii++)
    MemHandleUnlock(bigbitmapH[iii]);
}


void CreateBigBitmaps(void)
{
  CopyPart(0,10,24,0,0);
  CopyPart(10,4,8,2*48,40);
  CopyPart(14,2,48,0,40);
}

void FreeBigBitmaps(void)
{
  int iii;
  for (iii=0 ; iii<16 ; iii++)
    BmpDelete(bigbitmapP[iii]);
}

void SetPageResource(int set,int  num,int hour)
{
  int newid;
  int iii;
  BackgroundInfo *c;

  //if a theme is open...
  if (ThemeDB != 0)
  {
    //find current background id
    newid=-1;
    iii=0;
    c=&(ThemeDef->back);

    while ((newid < 0) && (iii<ThemeDef->count))
    {
      if ( (c->set == set) && (c->page == num) && (hour >= c->start ) && (hour < c->end))
      {
        newid=c->id;
      }
      iii++;
      c++;
    }
    if (newid < 0)
     newid=0;

    if (newid != BackgroundID)
    {
      changed[0]=1;
      changed[1]=1;

      //if a background is currently used...
      if (BackgroundH != 0)
        //release the handle
        MemHandleUnlock(BackgroundH);
      //use background on this page?
      if (newid != 0)
      {
        //yes, set the background
        BackgroundH=DmGetResource('Tbmp',newid);
        Background=MemHandleLock(BackgroundH);
      } else
      {
        //no, clear background
        BackgroundH=0;
        BackgroundID=0;
        Background=NULL;
      }
      BackgroundID=newid;
    }
  }
}

void OpenTheme(char *name)
{
  LocalID id;

  id=DmFindDatabase(0,name);
  if (id!=0)
  {
    ThemeDB=DmOpenDatabase(0,id,dmModeReadOnly);
    ThemeDefH=DmGetResource('ctrl',7168);
    ThemeDef=MemHandleLock(ThemeDefH);

    if (ThemeDef->numbers == 0)
       // use internal numbers
      CreateBigBitmaps();
    else
      // use theme numbers
      OpenBigBitmaps();
    SetPageResource(prefs.pageset,prefs.mode,Time[0].hour);
  } else
  {
    ThemeDB=0;
    Background=NULL;
    CreateBigBitmaps();
    prefs.theme[0]=0;
  }

}

void CloseTheme()
{
  if (ThemeDB != 0)
  {
    if (ThemeDef->numbers == 0)
      FreeBigBitmaps();
    else
      CloseBigBitmaps();

    MemHandleUnlock(ThemeDefH);
    if (BackgroundH != 0)
      MemHandleUnlock(BackgroundH);
    BackgroundH=0;
    BackgroundID=-1;
    DmCloseDatabase(ThemeDB);
    ThemeDB=0;
  }
  Background=NULL;
}


static int StartApplication(void)
{
  int error;
  int iii;
  int jjj;
  //int num;
  DmOpenRef BCdb=NULL;
  Word x;
  //VoidHand  hbmp;
  //BitmapPtr pbmp;
  //WinHandle oldwin;


  error = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);
  //usemidi=(romversion>=0x03003000);
  //usemidi=false;
  is35 = (romversion >= 0x03503000);
  //is35=false;
  if (is35)
    WinScreenMode(winScreenModeGetSupportsColor,NULL,NULL,NULL,&iscolor);
  else
    iscolor=false;



  SysCurAppDatabase(&cardNo, &appID);


  PrefGetPreferences(&SysPref);

  ReadPref();


  BCdb=OpenAlarm();
  if (BCdb==0) return 1;
  CloseAlarm(BCdb);

  //create a offscreen window and paint the numbers and symbols on it
  //later we copy parts of that image to display the data
  bitmaps=WinCreateOffscreenWindow(240,80,screenFormat,&x);
  if (bitmaps == NULL)
    return 1;


  //on this a panel display is created, after that copied to the screen
  build=WinCreateOffscreenWindow(160,62,screenFormat,&x);
  if (build == NULL)
    return 1;


  initbigbitmap();

  //init month and weekday strings
  SysCopyStringResource(daystrings,stringID_weekdays);
  SysCopyStringResource(monthstrings,stringID_months);

  iii=0;
  jjj=0;
  day[jjj]=&daystrings[iii];
  while (jjj<6)
  {
    if (daystrings[iii] == 32)
    {
     jjj++;
     daystrings[iii]=0;
     day[jjj]=&daystrings[iii+1];
    }
    iii++;
  }

  iii=0;
  jjj=0;
  month[jjj]=&monthstrings[iii];
  while (jjj<11)
  {
    if (monthstrings[iii] == 32)
    {
     jjj++;
     monthstrings[iii]=0;
     month[jjj]=&monthstrings[iii+1];
    }
    iii++;
  }

  SysCopyStringResource(detailweek,stringID_detailweek);
  SysCopyStringResource(detailday,stringID_detailday);
  SysCopyStringResource(detailhour,stringID_detailhour);

  for (iii=0; iii<4 ; iii++)
    SysCopyStringResource(stimer[iii],stringID_timerstart+iii);

  SysCopyStringResource(salarmmode[0],stringID_alarmmode2);
  SysCopyStringResource(salarmmode[1],stringID_alarmmode3);

  for (iii=0; iii<7 ; iii++)
    SysCopyStringResource(stimermode[iii],stringID_timermode0+iii);

  if (iscolor)
  {
    if (StrLen(prefs.theme) > 0)
      OpenTheme(prefs.theme);
    else
    {
      Background=NULL;
      CreateBigBitmaps();
    }
  } else
    Background=NULL;


  FrmGotoForm(formID_bigclock);
  return 0;
}


static void EventLoop(void)
{
  short err;
  int formID;
  FormPtr form;
  EventType event;
  int wait=0;
  RectangleType rc = {{0,0},{40,16}};

  do
  {

    EvtGetEvent(&event, wait);
    if (newyeardone==1)
      wait=0;
    else
      wait=10;

    if (SysHandleEvent(&event)) continue;
    if (MenuHandleEvent((void *)0, &event, &err)) continue;

    if (event.eType == frmLoadEvent)
    {
      formID = event.data.frmLoad.formID;
      form = FrmInitForm(formID);
      FrmSetActiveForm(form);
      switch (formID)
      {
      case formID_bigclock:
        FrmSetEventHandler(form, (FormEventHandlerPtr) bigclock);
         break;
      case formID_layout:
        FrmSetEventHandler(form, (FormEventHandlerPtr) layout);
        break;
      case formID_world:
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionsworld);
        break;
      case formID_global:
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionsglobal);
        break;
      case formID_localtime:
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionslocaltime);
        break;
      case formID_timer:
        ao=false;
        obase=wTimer1;
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionsalarm);
        break;
      case formID_alarm:
        ao=true;
        obase=wAlarm1;
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionsalarm);
        break;
      case formID_sound:
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionssound);
        break;
      case formID_color:
        FrmSetEventHandler(form, (FormEventHandlerPtr) optionscolor);
        break;
      case formID_history:
        FrmSetEventHandler(form, (FormEventHandlerPtr) labelhistory);
        break;

      case formID_newyear:
        FrmSetEventHandler(form, (FormEventHandlerPtr) newyear);
        break;

       default: break; 
      }
    }
    FrmDispatchEvent(&event);
  } while(event.eType != appStopEvent);
}

static void StopApplication(void)
{
  //int iii;

  //if (midilist != NULL)
  //  MemPtrFree(midilist);

  WritePref();

  WinDeleteWindow(bitmaps,true);
  WinDeleteWindow(build,true);

  if (iscolor)
  {
    //FreeBigBitmaps();
    if (StrLen(prefs.theme) > 0)
      CloseTheme();
    else
      FreeBigBitmaps();
  }


  FrmCloseAllForms ();
}


void SetPageNames()
{
  FormPtr   form;
  ControlPtr check;
  int iii;

  form = FrmGetActiveForm();

  for (iii=0; iii<4 ; iii++)
  {
    check = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,buttonID_m1+iii));
    CtlSetLabel(check,prefs.pagenames[prefs.pageset][iii]);
  }
}

// Display a big number
void DrawBigNumber(int num,int x, int y)
{
  RectangleType rc = {{24*num,0},{24,40}};
  if (iscolor)
    WinDrawBitmap(bigbitmapP[num],x,y);
  else
    WinCopyRectangle(bitmaps,WinGetDrawWindow(),&rc,x,y,scrCopy);
}

// Display AM or PM image
void DrawBigAMPM(int num,int x, int y)
{
  RectangleType rc = {{48*num,40},{48,40}};
  if (iscolor)
    WinDrawBitmap(bigbitmapP[num+14],x,y);
  else
    WinCopyRectangle(bitmaps,WinGetDrawWindow(),&rc,x,y,scrCopy);
}

// Display Big / : . *
void DrawBigSymbol(int num,int x, int y)
{
  RectangleType rc = {{2*48+8*num,40},{8,40}};
  if (iscolor)
    WinDrawBitmap(bigbitmapP[num+10],x,y);
  else
    WinCopyRectangle(bitmaps,WinGetDrawWindow(),&rc,x,y,scrCopy);
}

// Display a string in big
void DrawBig(char *text,int x, int y)
{
  while (*text != 0)
  {
    if ( (*text >= '0') && ( *text <= '9') )
    {
      DrawBigNumber(*text - '0',x,y);
      x+=24;
    } else
    {
      switch (*text)
      {
         case 'a' : DrawBigAMPM(0,x,y);
                    x+=48;
                    break;
         case 'p' : DrawBigAMPM(1,x,y);
                    x+=48;
                    break;
         case '#' : x+=24;
                    break;
         case '/' : DrawBigSymbol(0,x,y);
                    x+=8;
                    break;
         case ':' : DrawBigSymbol(1,x,y);
                    x+=8;
                    break;
         case '.' : DrawBigSymbol(2,x,y);
                    x+=8;
                    break;
         case '*' : DrawBigSymbol(3,x,y);
                    x+=8;
                    break;
         case ' ' : x+=8;
                    break;
         default  : break;           
      }
    }
    text++;
  }
}

// Draw large date with symbols from preferences
void DrawDate(int num,int y)
{
  int       year;
  int       dp;
  int       mp;
  int       yp;
  char      sep;
  char      date[8];
  char      sepchar[]={'/',':','.','*'};

  date[8]=0;

  //draw the weekday
  FntSetFont(2);
  MyDrawChars(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )],StrLen(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )]),0,y);

  //calculate the format
  sep=0;
  switch (SysPref.dateFormat)
  {
    case dfMDYWithSlashes: {
                             sep=0;
                             dp=3;
                             mp=0;
                             yp=6;
                             break;
                           }
    case dfDMYWithDashes: {
                             sep=3;
                             dp=0;
                             mp=3;
                             yp=6;
                             break;
                          }
    case dfDMYWithDots:  {
                             sep=2;
                             dp=0;
                             mp=3;
                             yp=6;
                             break;
                          }
    case dfDMYWithSlashes:
                           {
                             sep=0;
                             dp=0;
                             mp=3;
                             yp=6;
                             break;
                           }
    case dfYMDWithDashes: {
                             sep=3;
                             dp=6;
                             mp=3;
                             yp=0;
                             break;
                          }
    case dfYMDWithDots: {
                             sep=2;
                             dp=6;
                             mp=3;
                             yp=0;
                             break;
                         }
    case dfYMDWithSlashes:
                           {
                             sep=0;
                             dp=6;
                             mp=3;
                             yp=0;
                             break;
                           }
    default: break;                       
    }

    //symbol
    date[2]=sepchar[sep];
    date[5]=sepchar[sep];

    //Day
    date[dp]=(Time[num].day / 10) + '0';
    date[dp+1]=(Time[num].day % 10) + '0';

    //Month
    date[mp]=(Time[num].month / 10) + '0';
    date[mp+1]=(Time[num].month % 10) + '0';

    //Year
    year=Time[num].year % 100;   

    date[yp]=(year / 10) + '0';
    date[yp+1]=(year % 10) + '0';

    //Draw the date
    DrawBig(date,0,y+16);
}

//draw date in normal font above the time
void DrawSmallDate(int num,int y)
{
  char text[30];
  RectangleType rc={{80,y},{55,16}};

  DateToAscii(Time[num].month,Time[num].day,Time[num].year,SysPref.dateFormat,text);
  FntSetFont(2);
  //WinEraseRectangle(&rc,0);
  MyDrawChars(text,StrLen(text),80,y);
}


//draw the time
void DrawTime(int num,int y,Boolean alarm, Boolean timer,Boolean small)
{
  int h;
  int am;
  int sum;
  Boolean ShowDate;
  RectangleType rc;
  char time[9];
  Boolean special;
  unsigned char val1;
  unsigned char val2;
  char sep=':';

  MemSet(time,9,0);

  //display on set time screen?
  special=FrmGetActiveFormID() == formID_localtime;

  //display date above time?
  ShowDate=true;
  if (special)
     ShowDate=false;
  else
    {
      val1=prefs.pages[prefs.pageset][prefs.mode*2];
      val2=prefs.pages[prefs.pageset][prefs.mode*2+1];
      //upper and lower panel same worldtime
      if ((val1 & 7) == (val2 & 7))
      {
        //already a big date on upper or lower panel?
        sum= (val1 >> 3) + (val2 >> 3);
        if  ((sum==3) || (sum==10) || (sum==9))
          ShowDate=false;
      }
    }

  FntSetFont(2);

  // if no am/pm and seconds center it
  if ((prefs.world[num].options.seconds==0) && (prefs.world[num].options.ampm==0) && (!special))
    prefs.world[num].add=24;
  else
    prefs.world[num].add=0;

  if (alarm)
    prefs.world[num].add=56;

  am=0;
  h=Time[num].hour;
  if (
       (prefs.world[num].options.ampm==1) &&
       (prefs.world[num].options.trigger != 3)
      )
  {
    am=1;
    if (h>=12)
     am=2;
    if (h>12)
     h-=12;
  }

  switch (SysPref.timeFormat)
  {
    case tfDot: case  tfDotAMPM: case  tfDot24h :
    {
       sep='.';
    }
    default: break;
  }


  time[0]=(h / 10) + '0';
  time[1]=(h % 10) + '0';
  time[2]=sep;

  time[3]=(Time[num].minute / 10) + '0';
  time[4]=(Time[num].minute % 10) + '0';

  if ((prefs.world[num].options.seconds==1) || (special))
  {
    time[5]=sep;
    time[6]=(Time[num].second / 10) + '0';
    time[7]=(Time[num].second % 10) + '0';
  }

  if (am!=0)
  {
    if ( (prefs.world[num].options.seconds==0) && (!alarm) && (!special))
    {
      time[5]=' ';
      if (am==1)
        time[6]='a';
      else
        time[6]='p';
    } else
    {
      // small am/pm
      rc.extent.x =15;
      rc.extent.y = 16;
      rc.topLeft.x = 130;
      rc.topLeft.y = y;
      //WinEraseRectangle(&rc,0);
      if (am==1)
        MyDrawChars("am",2,140,y);
      else if (am==2)
        MyDrawChars("pm",2,140,y);
    }
  }
  if ((ShowDate) && (!alarm) && (!timer))
    DrawSmallDate(num,y);

   if (small)
     MyDrawChars(time,StrLen(time),0,y);
   else
    DrawBig(time,prefs.world[num].add,y+16);
}

void DrawAlarmCheck(int num, int y)
{
  Boolean	ignored;
  //char		ellipsisChar = 0x85; //horizEllipsisChr;
  SWord		itemTextLen;
  SWord		itemWidth;
  CharPtr	itemTextP;
  BigClockAlarm alarm;
  DmOpenRef     BCdb;
  char          number[8];
  RectangleType rc={{3,y+18},{20,20}};


  //WinEraseRectangle(&rc,0);
  WinDrawRectangleFrame(3,&rc);

  //remove checked if trigger=once and already done
  if (prefs.world[num].options.trigger==2)
    {
      BCdb=OpenAlarm();
      ReadAlarm(BCdb,num-5,&alarm);
      CloseAlarm(BCdb);
      if (alarm.options.active==0)
        prefs.world[num].options.active=0;
    }

   //draw the cross
  if (prefs.world[num].options.active==1)
  {
    WinDrawLine(3,y+18,23,y+38);
    WinDrawLine(3,y+19,22,y+38);
    WinDrawLine(4,y+18,23,y+37);

    WinDrawLine(23,y+18,3,y+38);
    WinDrawLine(22,y+18,3,y+37);
    WinDrawLine(23,y+19,4,y+38);
  }

  FntSetFont(0);
  if (prefs.world[num].options.base>0)
  {
    itemWidth = 32;
    itemTextLen = 160;
    itemTextP = prefs.world[wLocal+prefs.world[num].options.base].name;

    FntCharsInWidth(itemTextP, &itemWidth, &itemTextLen, &ignored);
    MyDrawChars(itemTextP, itemTextLen, 0,y+28+16);
  }
}


void DrawTime2(int y,int d,int h,int m,int s,Boolean small)
{
  char time [9];
  char sec[4];
  MemSet(time,9,0);

  time[2]='/';
  time[5]=':';
  while (d>=100)
    d-=100;

  time[0]=(d / 10) + '0';
  time[1]=(d % 10) + '0';
  time[3]=(h / 10) + '0';
  time[4]=(h % 10) + '0';
  time[6]=(m / 10) + '0';
  time[7]=(m % 10) + '0';

  if (small)
     MyDrawChars(time,StrLen(time),0,y);
  else
     DrawBig(time,0,y+16);

  if ((s>=0) != small)
  {
    FntSetFont(2);
    StrIToA(sec,s);
    if (s<10)
    {
      sec[2]=sec[1];
      sec[1]=sec[0];
      sec[0]='0';
    }
    MyDrawChars(sec,StrLen(sec),140,y);
  }
}


void DrawTimerTime(int num,int y,Boolean ns,Boolean small)
{
  long d,s,m,h,h2;
  unsigned long c;
  Boolean daymode;
  RectangleType rc={{140,y},{20,16}};
  char o[8];
  int iii,jjj,kkk;

  h2=prefs.world[num].value / hoursInSeconds;
  h=h2;
  if (h>=24)
  {
    d=(h / 24);
    h=h-d*24;
    daymode=true;
  } else
    daymode=false;

  m=(prefs.world[num].value - h2 * hoursInSeconds) / minutesInSeconds;
  s=prefs.world[num].value - h2 * hoursInSeconds - m * minutesInSeconds;

  if  (prefs.world[num].options.calc)
  {
    if (prefs.world[num].calc < 36000)
      c=  (  (unsigned long)prefs.world[num].value *
             (unsigned long)prefs.world[num].calc
           ) /
              (unsigned long)3600;
    else
      c=  (   (unsigned long)prefs.world[num].calc /
              (unsigned long)3600
           ) *
              (unsigned long)prefs.world[num].value;
    StrIToA(help,c);

    MemSet(&o,7,'0');
    o[7]=0;
    iii=StrLen(help);
    kkk=iii;
    jjj=6;
    while ((iii>0) && (jjj>=0))
    {
      if (jjj!=4)
      {
        o[jjj]=help[iii-1];
        iii--;
      }
      jjj--;
    }
    o[4]='.';

    if (small)
     MyDrawChars(o,7,0,y);
    else
      DrawBig(o,0,y+16);

  } else if (daymode)
  {
    if (((prefs.world[num].options.mode!=0) && (prefs.world[num].options.mode!=3)) || ns)
      s=-1;
    DrawTime2(y,d,h,m,s,small);
  }  else
  {

    Time[num].second=s;
    Time[num].minute=m;
    Time[num].hour=h;
    DrawTime(num,y,false,true,small);
  }

  if ((prefs.world[num].options.active==1) && (prefs.world[num].options.autooff==1))
    EvtResetAutoOffTimer();
}

void DrawTwoTimes(int num,int y)
{
  char text[30];
  RectangleType rc;
  int iii;

  for (iii=0 ; iii<2; iii++)
  {
    FntSetFont(1);
    MyDrawChars(prefs.world[num+iii].name,StrLen(prefs.world[num+iii].name),iii*80,y);
    FntSetFont(2);
    MyDrawChars(day[DayOfWeek( Time[num+iii].month, Time[num+iii].day, Time[num+iii].year )],StrLen(day[DayOfWeek( Time[num+iii].month, Time[num+iii].day, Time[num+iii].year )]),iii*80,y+14);
    DateToAscii(Time[num+iii].month,Time[num+iii].day,Time[num+iii].year,SysPref.dateFormat,text);
    MyDrawChars(text,StrLen(text),iii*80,y+28);
    TimeToAscii(Time[num+iii].hour,Time[num+iii].minute,SysPref.timeFormat,text);
    MyDrawChars(text,StrLen(text),iii*80,y+42);
  }
}

void DrawDateDetail(int num,int y)
{
  char text[30];
  char number[10];
  RectangleType rc;
  int  x1,x2,x3,xdraw,xdraw2;
  Long data[6];

  CalcDateData(Time[num],& data[0]);

  //Weekday
  FntSetFont(2);
  MyDrawChars(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )],StrLen(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )]),0,y);

  //Date
  DateToAscii(Time[num].month,Time[num].day,Time[num].year,SysPref.dateFormat,text);
  MyDrawChars(text,StrLen(text),80,y);

  x1=FntCharsWidth(detailweek,StrLen(detailweek));
  x2=FntCharsWidth(detailweek,StrLen(detailday));
  x3=FntCharsWidth(detailweek,StrLen(detailhour));
  if (x1>x2)
  {
    if (x1>x3)
      xdraw=x1;
    else
      xdraw=x3;
  } else
  {
    if (x2>x3)
      xdraw=x2;
    else
      xdraw=x3;
  }
  xdraw+=8;
  //StrIToA(number,hourleft);
  StrIToA(number,data[5]);
  xdraw2=FntCharsWidth(number,StrLen(number))+16+xdraw;

  //Hour
  MyDrawChars(detailhour,StrLen(detailhour),0,y+42);
  //StrIToA(text,hour);
  StrIToA(text,data[4]);
  MyDrawChars(text,StrLen(text),xdraw,y+42);

  StrCopy(text,"(");
  StrCat(text,number);
  StrCat(text,")");
  MyDrawChars(text,StrLen(text),xdraw2,y+42);

  //Week
  MyDrawChars(detailweek,StrLen(detailweek),0,y+14);
  StrIToA(text,data[0]);
  MyDrawChars(text,StrLen(text),xdraw,y+14);

  StrIToA(number,data[1]);
  StrCopy(text,"(");
  StrCat(text,number);
  StrCat(text,")");
  MyDrawChars(text,StrLen(text),xdraw2,y+14);

  //Day
  MyDrawChars(detailday,StrLen(detailday),0,y+28);
  StrIToA(text,data[2]);
  MyDrawChars(text,StrLen(text),xdraw,y+28);

  StrIToA(number,data[3]);
  StrCopy(text,"(");
  StrCat(text,number);
  StrCat(text,")");
  MyDrawChars(text,StrLen(text),xdraw2,y+28);

}


void ColorSet(Boolean i)
{
  if (iscolor)
  {
    if (i)
    {
      WinSetTextColor(prefs.colorfi);
      WinSetBackColor(prefs.colorbi);
      WinSetForeColor(prefs.colorbi);
    } else
    {
      WinSetBackColor(prefs.colorb);
      WinSetForeColor(prefs.colorb);
      WinSetTextColor(prefs.colorf);
    }
  }
}

void DrawAlarmDays(int num,int x,int y)
{
  int iii;
  unsigned char bits[]={1,2,4,8,16,32,64,128,1};
  RectangleType rc={{x-1,y},{56,11}};
  Boolean i;
  int w;


  rc.extent.x = 8;
  FntSetFont(0);
  for (iii=0 ; iii<7 ; iii++)
  {
    rc.topLeft.x = x+iii*8-1;

    i=(prefs.world[num].days & bits[(iii+SysPref.weekStartDay) % 7]) != 0;
    if (iscolor)
    {
      ColorSet(i);
      if (i)
        WinDrawRectangle(&rc,0);
    }

    w=FntCharWidth(day[(iii+SysPref.weekStartDay) % 7][0]);
    MyDrawChars(day[(iii+SysPref.weekStartDay) % 7],1,x+iii*8+3-(w /2),y);
    if (i && (!iscolor))
      WinInvertRectangle(&rc,0);
  }

}

void DrawAlarmSound(int num,int x,int y)
{
  int iii;
  RectangleType rc={{x,0},{19,9}};
  char c= '1';

  FntSetFont(1);

  for (iii=0; iii<4; iii++)
  {
    rc.topLeft.y = 16+y+iii*10;

    if (iscolor)
    {
      ColorSet(prefs.world[num].options.sound==iii);
      if (prefs.world[num].options.sound==iii)
        WinDrawRectangle(&rc,0);
      MyDrawChars(&c,1,x+7,16+y+iii*10-1);
      WinSetForeColor(prefs.colorf);
      WinDrawRectangleFrame(1,&rc);
    } else
    {
      rc.topLeft.y = 16+y+iii*10;
      WinEraseRectangle(&rc,0);
      MyDrawChars(&c,1,x+7,16+y+iii*10-1);
      WinDrawRectangleFrame(1,&rc);
      if (prefs.world[num].options.sound==iii)
        WinInvertRectangle(&rc,0);
    }
    c++;
  }
}

void DrawButton(char *t,Boolean i,int x, int y, int h, int w, int o)
{
  RectangleType rc={{x,y},{w,h}};
  FntSetFont(2);

  if (iscolor)
  {
    ColorSet(i);
    if (i)
      WinDrawRectangle(&rc,5);
    MyDrawChars(t,StrLen(t),(w / 2) - (FntCharsWidth(t,StrLen(t)) / 2) + x,y+o);

    WinSetForeColor(prefs.colorf);
    WinDrawRectangleFrame(roundFrame,&rc);

  } else
  {
    if (i)
      {
        WinDrawRectangleFrame(roundFrame,&rc);
        rc.extent.x =w+2;
        rc.extent.y = h+2;
        rc.topLeft.x = x-1;
        rc.topLeft.y = y-1;
        WinDrawRectangle(&rc,7);
        WinEraseChars(t,StrLen(t),(w/2)-(FntCharsWidth(t,StrLen(t)) / 2)+x,y+o);
      }
    else
      {
        WinEraseRectangle(&rc,0);
        WinDrawRectangleFrame(roundFrame,&rc);
        MyDrawChars(t,StrLen(t),(w/2)-(FntCharsWidth(t,StrLen(t)) / 2)+x,y+o);
     }
  }

}


void DrawUpDown(int num,int y)
{
  RectangleType rc={{115,0},{19,19}};
  int iii;
  char  d;
  Boolean i;

  FntSetFont(5);
  for (iii=0; iii<2; iii++)
  {
    rc.topLeft.y = y+16+iii*20;
    d=iii+1;

    i=iii != prefs.world[num].options.timerup;
    if (iscolor)
    {
      ColorSet(i);
      if (i)
        WinDrawRectangle(&rc,0);
      MyDrawChars(&d,1,112+7,y+20+iii*22);
      WinSetForeColor(prefs.colorf);
      WinDrawRectangleFrame(1,&rc);
    } else
    {
      WinEraseRectangle(&rc,0);
      WinDrawRectangleFrame(1,&rc);
      MyDrawChars(&d,1,112+7,y+20+iii*22);
      if (i)
        WinInvertRectangle(&rc,0);
    }
  }
}

void DrawTimerControls(int num,int y)
{
  RectangleType rc={{115,0},{19,19}};
  char  d;
  int iii;
  char text[25];
  DateTimeType rel;
  Boolean i;

  FntSetFont(2);

  DrawButton(stimer[prefs.world[num].options.active],prefs.world[num].options.active,3,y+16,39,43,13);
  DrawButton(stimer[2],timercommand==1+10*num,59,y+16,17,43,2);
  DrawButton(stimer[3],timercommand==2+10*num,59,y+16+22,17,43,2);

  DrawUpDown(num,y);
  DrawAlarmSound(num,139,y);

  if (prefs.world[num].options.daterelative==1)
  {
    StrCopy(text,"");
    FntSetFont(1);
    TimSecondsToDateTime(prefs.world[num].date,&rel);
    DateToAscii(rel.month,rel.day,rel.year,SysPref.dateFormat,help);
    StrCat(text,help);
    TimeToAscii(rel.hour,rel.minute,SysPref.timeFormat,help);
    StrCat(text," - ");
    StrCat(text,help);
    MyDrawChars(text,StrLen(text),0,y+2);
  }  else if (prefs.world[num].options.calc==1)
  {
    if (prefs.world[num].calc > 100)
    {
      StrIToA(text,prefs.world[num].calc / 100);
      StrCat(text,".");
    }
    else
      StrCopy(text,("0."));

    StrIToA(help,prefs.world[num].calc % 100);
    if (StrLen(help)==2)
      StrCat(text,help);
    else
    {
      StrCat(text,"0");
      StrCat(text,help);
    }

    StrCat(text," / hour");
    FntSetFont(1);
    MyDrawChars(text,StrLen(text),0,y+2);
  }
}

void DrawTiny(int x,int y,int n)
{
  if (n & 1)
    WinDrawLine(x,y,x+2,y);
  if (n & 2)
    WinDrawLine(x,y,x,y+2);
  if (n & 4)
    WinDrawLine(x+2,y,x+2,y+2);
  if (n & 8)
    WinDrawLine(x,y+2,x+2,y+2);
  if (n & 16)
    WinDrawLine(x,y+2,x,y+4);
  if (n & 32)
    WinDrawLine(x+2,y+2,x+2,y+4);
  if (n & 64)
    WinDrawLine(x,y+4,x+2,y+4);
}

void TinyNumber(int x,int y,int num)
{
  const numbers[10] = {119,36,93,109,46,107,123,37,127,111};
  //if ((num >=0) && (num<=9))
  DrawTiny(x,y,numbers[num ]);
}

/*
void DrawSysInfo(int y)
{
  ULong free;
  int id;
  ULong s;
  ULong m;
  char help[16];
  char help2[16];
  MemPtr startPP;
  MemPtr endPP;
  WinHandle wh;
  WinPtr  wp;
  char *screen;
  int iii;
  DWord romversion;
  int error;
  unsigned char *c;


  FntSetFont(0);


  error = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romversion);

  c=(char *)&romversion;
  MyDrawChars("OS:",3,0,y);
  StrCopy(help,"V ");
  iii=*c;
  StrIToA(help2,iii);
  StrCat(help,help2);
  StrCat(help,".");
  c++;
  s=*c;
  StrIToH(help2,s);
  StrCat(help,&help2[6]);
  MyDrawChars(help,StrLen(help),50,y);



  MyDrawChars("Freemem:",8,0,y+1*12);
  MemCardInfo(0,NULL,NULL,NULL,NULL,NULL,NULL,&free);
  StrIToA(help,free);
  MyDrawChars(help,StrLen(help),50,y+1*12);

  MyDrawChars("Dynamic:",8,0,y+2*12);

  id=MemHeapID(0,0);
  if (MemHeapDynamic(id))
  {
    MemHeapFreeBytes(id,&s,&m);
    StrIToA(help,s);
    MyDrawChars(help,StrLen(help),50,y+2*12);
  }
  if (romversion  >= 0x03003000)
  {
      MyDrawChars("Stack:",6,0,y+3*12);
      SysGetStackInfo(&startPP,&endPP);
      StrIToA(help,(Long)endPP-(Long)startPP);
      MyDrawChars(help,StrLen(help),50,y+3*12);
  }

}*/


void DrawMonthView(int num,int y,Boolean r)
{
  int iii;
  int jjj;
  int kkk;
  int d;
  int s;
  RectangleType rc;
  char help2[5];
  char sym[2];
  char daytext[4];
  int mo;
  int yo;
  DateTimeType old;
  Long data[6];
  DateTimeType week;
  Boolean wd;
  int w;


  TimSecondsToDateTime(prefs.world[num].last,&old);

  FntSetFont(1);
  for (iii=0 ; iii<7 ; iii++)
  {
    w=FntCharWidth(day[(iii+SysPref.weekStartDay) % 7][0]);
    MyDrawChars(day[(iii+SysPref.weekStartDay) % 7],1,3+iii*11+5-(w /2),y);
  }

  mo=((Time[num].month-1)+prefs.world[num].monthoffset+12*100) % 12;
  if (prefs.world[num].monthoffset<0)
    yo=Time[num].year+( Time[num].month+prefs.world[num].monthoffset-12) / 12;
  else
    yo=Time[num].year+( Time[num].month+prefs.world[num].monthoffset-1) / 12;

  d=DaysInMonth(mo+1,yo);
  s=DayOfWeek(mo+1,1,yo);
  jjj=(s-SysPref.weekStartDay+7) % 7;
  kkk=1;
  iii=0;

  week=Time[num];
  week.month=mo+1;
  week.year=yo;
  wd=false;

  rc.extent.x =9;
  rc.extent.y = 7;
  while ((iii<7) && (kkk<=d))
  {
    rc.topLeft.x = 3+jjj*11;
    rc.topLeft.y = iii*6+y+11;

    if ((kkk==Time[num].day) && (yo==Time[num].year) && ((mo+1)==Time[num].month))
      {
        if (iscolor)
        {
          WinSetForeColor(prefs.colorbi);
          WinDrawRectangle(&rc,0);
        } else
          WinEraseRectangle(&rc,0);
      }

   if (iscolor)
   {
      if ((kkk==Time[num].day) && (yo==Time[num].year) && ((mo+1)==Time[num].month))
      {
        WinSetForeColor(prefs.colorfi);
      } else
      {
        WinSetForeColor(prefs.colorf);
      }

   }

    if ((kkk/10) != 0)
    {
      TinyNumber(4+jjj*11,iii*6+y+12,kkk / 10);
      TinyNumber(4+jjj*11+4,iii*6+y+12,kkk % 10);
    } else
      TinyNumber(4+jjj*11+2,iii*6+y+12,kkk % 10);

    if (! iscolor)
      if ((kkk==Time[num].day) && (yo==Time[num].year) && ((mo+1)==Time[num].month))
        WinInvertRectangle(&rc,0);

    if ((! wd) && (jjj>0))
    {
      wd=true;
      week.day=kkk;
      CalcDateData(week,&data[0]);
      if ((data[0] / 10) != 0)
      {
        TinyNumber(92-4,iii*6+y+12,data[0] / 10);
        TinyNumber(92,iii*6+y+12,data[0] % 10);
      } else
        TinyNumber(92,iii*6+y+12,data[0] % 10);

    }

    jjj++;
    if (jjj>=7)
      {
        wd=false;
        iii++;
        jjj=0;
      }
    kkk++;
  }
  FntSetFont(0);
  StrCopy(help,month[mo]);
  StrIToA(help2,yo);
  StrCat(help," ");
  StrCat(help,help2);
  iii=FntCharsWidth(help,StrLen(help));
  MyDrawChars( help,StrLen(help),40-(iii /2),y+48);

  FntSetFont(2);
  StrCopy(help,month[Time[num].month-1]);
  iii=FntCharsWidth(help,StrLen(help));
  MyDrawChars( help,StrLen(help),160-iii,y);

  if ((Time[num].day / 10) != 0)
    daytext[0]=(Time[num].day / 10) + '0';
  else
    daytext[0]='#';
  daytext[1]=(Time[num].day % 10) + '0';
  daytext[2]=0;

  DrawBig(daytext,112,y+16);

}

void SwitchDrawSection(int val,int y,Boolean sw)
{
  int iii;
  unsigned char pan;
  unsigned char num;
  char text[64];
  char help[32];
  DateTimeType rel;
  WinHandle old;
  RectangleType all = {{0,0},{160,62}};
  RectangleType part = {{80,0},{80,62}};
  RectangleType test = {{1,32},{158,8}};
  Int16 width;
  Int16 length;
  Boolean trunc;
  VoidHand  hbmp;
  BitmapPtr pbmp;
  int id;
  ULong s;
  ULong m;


  old=WinGetDrawWindow();
  WinSetDrawWindow(build);

  if (iscolor)
  {
    WinSetForeColor(prefs.colorf);
    WinSetTextColor(prefs.colorf);
    WinSetBackColor(prefs.colorb);

    if (Background != NULL)
    {
      if (y==ytop)
        WinDrawBitmap(Background,0,0);
      else
        WinDrawBitmap(Background,0,-62);
    } else
      WinEraseWindow();

  }  else
  {
    WinEraseWindow();
  }

  pan=val >> 3;
  num= val & 7;
  //sw=false;

  switch (pan)
  {
    case pnEmpty: {
                     /*
                     id=MemHeapID(0,0);
                      if (MemHeapDynamic(id))
                      {
                        MemHeapFreeBytes(id,&s,&m);
                        StrIToA(help,s);
                        WinDrawChars(help,StrLen(help),10,y+2);
                      }*/
                    break;
                  }
    case pnDate: {
              DrawDate(wLocal+num,0);
              break;
            }
    case pnTime:  {
              FntSetFont(2);
              width=80;
              length=11;
              FntCharsInWidth(prefs.world[wLocal+num].name,&width,&length,&trunc);
              MyDrawChars(prefs.world[wLocal+num].name,length,0,ymove);
              DrawTime(wLocal+num,ymove,false,false,false);
              break;
            }
    case pnAlarm: {
              TimSecondsToDateTime(prefs.world[wAlarm1+num].value,&Time[wAlarm1+num]);
              FntSetFont(2);

              width=80;
              length=11;
              FntCharsInWidth(prefs.world[wAlarm1+num].name,&width,&length,&trunc);
              MyDrawChars(prefs.world[wAlarm1+num].name,length,0,ymove);

              DrawAlarmCheck(wAlarm1+num,ymove);
              DrawTime(wAlarm1+num,ymove,true,false,false);

              if (prefs.world[wAlarm1+num].options.trigger==1)
                DrawAlarmDays(wAlarm1+num,81,ymove);
              else
                MyDrawChars(salarmmode[prefs.world[wAlarm1+num].options.trigger-2],StrLen(salarmmode[prefs.world[wAlarm1+num].options.trigger-2]),81,ymove);

              DrawAlarmSound(wAlarm1+num,34,ymove);
              break;
            }
    case pnTimer: case pnCTimer:{
              FntSetFont(2);

              width=80;
              length=11;
              FntCharsInWidth(prefs.world[wTimer1+num].name,&width,&length,&trunc);
              MyDrawChars(prefs.world[wTimer1+num].name,length,0,ymove);

              if (pan==pnTimer)
                DrawTimerTime(wTimer1+num,ymove,false,false);
              else
              {
                DrawTimerTime(wTimer1+num,ymove+18,false,true);
                DrawButton(stimer[prefs.world[wTimer1+num].options.active],prefs.world[wTimer1+num].options.active,3,ymove+22+16,17,43,2);
                DrawButton(stimer[2],timercommand==1+10*(wTimer1+num),59,ymove+16,17,43,2);
                DrawButton(stimer[3],timercommand==2+10*(wTimer1+num),59,ymove+22+16,17,43,2);
                DrawUpDown(wTimer1+num,ymove);
                DrawAlarmSound(wTimer1+num,139,ymove);
              }
              ColorSet(false);
              iii=prefs.world[wTimer1+num].options.mode;
              if (iii>6)
              {
                prefs.world[wTimer1+num].options.mode=3;
                iii=3;
              }
              FntSetFont(2);
              MyDrawChars(stimermode[iii],StrLen(stimermode[iii]),81,ymove);
              break;
            }
    case pnVTimer: {
                    StrCopy(text,prefs.world[wTimer1+num].name);
                    if (prefs.world[wTimer1+num].options.daterelative==1)
                    {
                      StrCat(text," - ");
                      FntSetFont(1);
                      TimSecondsToDateTime(prefs.world[wTimer1+num].date,&rel);
                      DateToAscii(rel.month,rel.day,rel.year,SysPref.dateFormat,help);
                      StrCat(text,help);
                      TimeToAscii(rel.hour,rel.minute,SysPref.timeFormat,help);
                      StrCat(text," - ");
                      StrCat(text,help);

                    }
                    FntSetFont(1);
                    iii=FntCharsWidth(text,StrLen(text));
                    MyDrawChars(text,StrLen(text),80-(iii/2),ymove+2);
                    DrawTimerTime(wTimer1+num,ymove,true,false);
                      break;
                   }
    case pnControl:{
               DrawTimerControls(wTimer1+num,ymove);
               break;
             }
    case pnWorld: {
               DrawTwoTimes(wLocal+num*2+1,ymove);
               break;
             }
    case pnDateD: {
               DrawDateDetail(wLocal+num,ymove);
               break;
             }
    case pnMonth:
              DrawMonthView(wLocal+num,ymove,true);
              break;
    case pnAnalog:
              DrawAnalogTime(wLocal+num,ymove);
              break;
    default: break;


  }

  WinSetDrawWindow(old);
  WinCopyRectangle(build,old,&all,0,y,scrCopy);
}


void IdleDrawSection(int val,int y,int up)
{
  if (changed[up] != 0)
  {
    SwitchDrawSection(val,y,false);
    changed[up]=0;
  }
}


int checkchanged(int pan)
{
  int r=0;

  switch (pan)
  {
    case pnDate :
    case pnMonth :
       if (prefs.world[0].options.daychanged == 1)
         r=1;
      break;

    case pnTime :
    case pnWorld :
    case pnDateD :
    case pnAnalog :
    case pnTimer :
    case pnCTimer :
    case pnVTimer :
      r=1;
      break;
  }
  return r;
}

int UpdateTimes()
{
  int iii;
  int n;
  Long now;
  Long add;
  BigClockAlarm alarm;
  DateTimeType last;
  BigClockTime *bct;
  int result=0;
  unsigned char pan;

  now=TimGetSeconds();

  TimSecondsToDateTime(now,&Time[0]);
  TimAdjust(&Time[0],prefs.world[0].value);

  TimSecondsToDateTime(prefs.world[0].last,&last);
  if (
       (last.year != Time[0].year) ||
       (last.month != Time[0].month) ||
       (last.day != Time[0].day)
      )
    prefs.world[0].options.daychanged=1;
  else
    prefs.world[0].options.daychanged=0;

  if (last.hour != Time[0].hour)
    SetPageResource(prefs.pageset,prefs.mode,Time[0].hour);


  for (iii=0 ; iii<4 ; iii++)
  {
    //adjust time
    Time[iii+1]=Time[0];
    TimAdjust(&Time[iii+1],prefs.world[iii+1].value);

    TimSecondsToDateTime(prefs.world[iii+1].last,&last);
    if (
         (last.year != Time[iii+1].year) ||
         (last.month != Time[iii+1].month) ||
         (last.day != Time[iii+1].day)
      )
      prefs.world[iii+1].options.daychanged=1;
    else
      prefs.world[iii+1].options.daychanged=0;


    prefs.world[iii+1].last=TimDateTimeToSeconds(&Time[iii+1]);


    bct=&prefs.world[wTimer1+iii];
    //adjust timers
    if (bct->options.active)
    {
      if (bct->options.daterelative==0)
      {
        if (bct->last !=0)
        {
          add=now-bct->last;
          bct->last=now;
        } else
        {
          add=1;
          bct->last=now;
        }

        if (bct->options.timerup == 0)
          add=-add;


        bct->value=bct->value+add;
      } else
      {
        bct->value=now-bct->date;
        if (bct->value < 0)
        {
          bct->value=-bct->value;
          if ( bct->options.timerup!=0)
          {
             bct->options.timerup=0;
             result=2;
          }
        }  else
        {
          if (bct->options.timerup != 1 )
          {
            bct->options.timerup=1;
            result=2;
          }
        }
      } // else date relative

      if (bct->value<0)
      {
        if ((bct->options.mode < 5) && (bct->options.mode!=2) && (bct->options.mode!=4))
          bct->value=0;
        add=0;
      }

      if ((bct->value<=0) && (bct->options.timerup == 0) /*(add<=0)*/)
      {
        if ((bct->options.mode==2) || (bct->options.mode==4))
        {
          while (bct->value <= 0)
            bct->value+=bct->reset;
        }
        else if ((bct->options.mode>0) && (bct->options.mode < 5))
          bct->value=bct->reset;
        if ((bct->options.mode != 2) && (bct->options.mode < 4))
          bct->options.active=false;
        if (bct->options.mode == 4)
          UpdateTimer(wTimer1+iii);
        if (bct->options.mode >= 5)
        {
          bct->value=-bct->value;
          bct->options.timerup=1;
        }
        //SetNextAlarm(cardNo,appID);

        prefs.world[0].last=now;
        SwitchDrawSection(prefs.pages[prefs.pageset][prefs.mode*2],ytop,true);
        SwitchDrawSection(prefs.pages[prefs.pageset][prefs.mode*2+1],ybottom,true);
        return 0;
      }
    }
  }

  if (
       (Time[0].month==1) &&
       (Time[0].day==1) &&
       (Time[0].hour==0) &&
       (Time[0].minute==0)
      )
  {
    if (newyeardone==0)
      FrmGotoForm(formID_newyear);
  }


  if (prefs.world[0].last != now )
  {
    prefs.world[0].last=now;
    result++;

    //does panel require update?
    n=checkchanged((prefs.pages[prefs.pageset][prefs.mode*2]) >> 3);
    if (n>0)
      changed[0]=n;
    n=checkchanged((prefs.pages[prefs.pageset][prefs.mode*2+1]) >> 3);
    if (n>0)
      changed[1]=n;
  }
  return result;
}


void SwitchDraw()
{
  UpdateTimes();
  SwitchDrawSection(prefs.pages[prefs.pageset][prefs.mode*2],ytop,true);
  SwitchDrawSection(prefs.pages[prefs.pageset][prefs.mode*2+1],ybottom,true);
}

void DrawCaptionTime()
{
  char text[30];
  RectangleType rc = {{115,0},{45,12}};
  int w;

  TimeToAscii(Time[0].hour,Time[0].minute,SysPref.timeFormat,text);
  FntSetFont(1);

  if (iscolor)
  {
    WinSetForeColor(UIColorGetTableEntryIndex(UIFormFill));
    WinSetBackColor(UIColorGetTableEntryIndex(UIFormFill));
    WinSetTextColor(UIColorGetTableEntryIndex(UIFieldText));
  }

  WinEraseRectangle(&rc,0);
  w=FntCharsWidth(text,StrLen(text));
  MyDrawChars(text,StrLen(text),160-w,1);
}

void IdleDraw()
{
  int n;

  n=UpdateTimes();
  if (n>0)
  {
    DrawCaptionTime();
    if (n>1)
      SwitchDraw();
    else
      {
        IdleDrawSection(prefs.pages[prefs.pageset][prefs.mode*2],ytop,0);
        IdleDrawSection(prefs.pages[prefs.pageset][prefs.mode*2+1],ybottom,1);
      }
  }
}

void DrawAnalogTime(int num,int y)
{
  int iii;
  WinHandle clock;
  Word error;
  WinHandle old;
  RectangleType all = {0,0,57,57};
  char daytext[4];

  DrawCircle(28,28+analogx,28+analogy);
  DrawCircle(26,28+analogx,28+analogy);
  DrawCircle(4,28+analogx,28+analogy);

  for (iii=0 ; iii< 12 ; iii++)
     WLine(iii*5,27,27,28+analogx,28+analogy);


  DrawZeiger((Time[num].hour % 12) *5 + (Time[num].minute / 12), 15, 5,28+analogx,28+analogy);
  DrawZeiger(Time[num].minute, 22, 5,28+analogx,28+analogy);
  if (prefs.world[num].options.seconds==1)
    DrawZeiger(Time[num].second, 25, 5,28+analogx,28+analogy);

  FntSetFont(1);
  StrCopy(help,prefs.world[num].name);
  MyDrawChars( help,StrLen(help),8 ,y);
  FntSetFont(2);

  MyDrawChars(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )],StrLen(day[DayOfWeek( Time[num].month, Time[num].day, Time[num].year )]),8,y+14);
  DateToAscii(Time[num].month,Time[num].day,Time[num].year,SysPref.dateFormat,help);
  MyDrawChars(help,StrLen(help),8,y+28);


}


void ChangeDays(int num, int d)
{
  unsigned char b;
  if (prefs.world[num].options.trigger==1)
  {
    b=prefs.world[num].days;
    b=b ^ ( 1 << ((d+SysPref.weekStartDay) % 7));
    prefs.world[num].days=b;
  }

}

void ChangeAlarm(int num , int d , Boolean up,int ten)
{
  long add;
  DateTimeType temp;

  //TimSecondsToDateTime(prefs.world[num].value,&temp);

  if (!up)
    add=-1;
  else
    add=1;

  switch (d)
  {
    case 5: add=1;
            break;
    case 4: add=10;
            break;
    case 3: add=60;
            break;
    case 2: add=600;
            break;
    case 1: add=hoursInSeconds;
            break;
    case 0: add=(long)hoursInSeconds*ten;
            break;
    case -1: add=(long)hoursInSeconds * 24;
             break;
    case -2: add=(long)hoursInSeconds * 24 *10;
             break;

  }

  if (!up)
    add=-add;

  //TimAdjust(&temp,add);

  //prefs.world[num].value=TimDateTimeToSeconds(&temp);
  prefs.world[num].value=prefs.world[num].value+add;//TimDateTimeToSeconds(&temp);
}

void ChangeCalcTimer(int num , int d , Boolean up)
{
  long add;

  if (!up)
    add=-1;
  else
    add=1;

  switch (d)
  {
    case 0: add=(long)3600 / (long)prefs.world[num].calc;
            break;
    case 1: add=(long)36000 / (long)prefs.world[num].calc ;
            break;
    case 2: add=(long)360000 / (long)prefs.world[num].calc;
            break;
    case 3: add=(long)3600000 / (long)prefs.world[num].calc;
            break;
    case 4: add=(long)36000000 / (long)prefs.world[num].calc;
            break;
    case 5: add=(long)360000000 / (long)prefs.world[num].calc;
            break;
  }

  if (!up)
    add=-add;

  //TimAdjust(&temp,add);

  prefs.world[num].value=prefs.world[num].value+add;//TimDateTimeToSeconds(&temp);
}


/*
//for debugging
void Showdate(char *name,unsigned long d)
{
  DateTimeType s;
  char h1[32];
  char h2[32];

  TimSecondsToDateTime(d,&s);
  DateToAscii(s.month,s.day,s.year,dfDMYWithDots,h1);
  TimeToAscii(s.hour,s.minute,false,h2);
  FrmCustomAlert(alertID_text3,name,h1,h2);
};*/


void SetNextAlarm(UInt cno,LocalID aid)
{
  unsigned long as;
  unsigned long now;
  unsigned long al;
  TimeType t;
  DateTimeType d;
  int num;
  BigClockAlarm *alarm;
  int iii;
  DmOpenRef BCdb=NULL;
  Boolean wakeup;
  char number[16];


  alarm=MemPtrNew(sizeof(BigClockAlarm));


  wakeup=false;
  BCdb=OpenAlarm();

  as=0;
  now=TimGetSeconds();
  num=-1;

  for (iii=0; iii<aCount; iii++)
  {
    ReadAlarm(BCdb,iii,alarm);
    //StrIToA(number,alarm->snoozecurrent);
    //FrmCustomAlert(alertID_text,number,NULL,NULL);

    if ((alarm->options.active) || (alarm->snoozecurrent > 0))
    {
      if (alarm->snoozecurrent > 0 )
      {
        al=alarm->snoozeoffset;
      } else
      {
        //set now,d to worldtime of alarm
        now=TimGetSeconds();
        TimSecondsToDateTime(now,&d);
        if (alarm->options.trigger !=3)
          TimAdjust(&d,alarm->worldoffset);
        now=TimDateTimeToSeconds(&d);

        al=0;
        wakeup=false;

        if ((alarm->days<128) && (alarm->days>0))
        {
          //normal alarm
          d.hour=alarm->time.hour;
          d.minute=alarm->time.minute;
          d.second=0;
          al=TimDateTimeToSeconds(&d);


          while (
                 ( (alarm->days & ( 1 << d.weekDay)) == 0 ) ||
                   ( al <= now)
                 )
          {
            al+=daysInSeconds;
            TimSecondsToDateTime(al,&d);
          }

          //adjust alarm time to local time
          if (alarm->options.trigger !=3)
            TimAdjust(&d,-alarm->worldoffset);
          al=TimDateTimeToSeconds(&d);
          //Showdate("alarm adjusted",al);


        } else if (alarm->days>=128)
        {
          if (alarm->options.trigger != 3)
          {
             //timer alarm
             al=TimDateTimeToSeconds(&(alarm->time));
             if (al > (now+60))
             {
               al-=60;
               wakeup=true;
             }
          }
          else
            {
              //chime
              if (( TimDateTimeToSeconds(&alarm->time) % daysInSeconds) != 0)
              {
                al=now / ( TimDateTimeToSeconds(&alarm->time) % daysInSeconds);
                al=(al + 1) * (TimDateTimeToSeconds(&alarm->time) % daysInSeconds);

                //adjust to local time
                TimSecondsToDateTime(al,&d);
                if (alarm->options.trigger !=3)
                  TimAdjust(&d,-alarm->worldoffset);
                al=TimDateTimeToSeconds(&d);
              } else
                al=0;
            }
        }
      }

      //set now to local time
      now=TimGetSeconds();

      if (
           (
            ( al>(now+1)) &&

            (
               (al<as) ||
               (as==0)
            )
           ) ||
           (
            ( al>(now+1)) &&
            ( alarm->options.trigger != 3) &&
            (
               (al<=as) ||
               (as==0)
            )
           )
         )
        {

          as=al;
          if (wakeup)
            num=-2;
          else
            num=iii;

        }
    }
  }


  if (num==-1)
    as=0;
  CloseAlarm(BCdb);

  AlmSetAlarm(cno,aid,num,as,false);

  MemPtrFree(alarm);

}

void SetSound(int a,int w)
{
  int num;
  if ((w>=0) && (w<=3))
  {
    prefs.world[a].options.sound=w;
  }
}


void UpdateAlarm(int num)
{
  BigClockAlarm alarm;
  DmOpenRef BCdb=NULL;

  BCdb=OpenAlarm();

  ReadAlarm(BCdb,num-5,&alarm);

  TimSecondsToDateTime(prefs.world[num].value,&alarm.time);

  alarm.options=prefs.world[num].options;
  alarm.options.ignoresystemsoundoff=prefs.ignoresystemsoundoff;
  alarm.sound=prefs.sounds[prefs.world[num].options.sound];
  alarm.snoozecurrent=0;
  alarm.snoozecount=prefs.world[num].snoozecount;
  alarm.snoozetime=prefs.world[num].snoozetime;

  StrCopy(alarm.name,prefs.world[num].name);
  StrCopy(alarm.worldname,prefs.world[prefs.world[num].options.base].name);
  alarm.worldoffset=prefs.world[prefs.world[num].options.base].value;
  if (alarm.options.trigger==1)
    alarm.days=prefs.world[num].days;
  else if (alarm.options.trigger==0)
    alarm.days=127;
  else if (alarm.options.trigger==2)
    alarm.days=127;
  else if (alarm.options.trigger==3)
    alarm.days=128;

  WriteAlarm(BCdb,num-5,&alarm);
  CloseAlarm(BCdb);
}

void ToggleAlarm(int num)
{
  prefs.world[num].options.active = inot[prefs.world[num].options.active];
  UpdateAlarm(num);
  SetNextAlarm(cardNo,appID);
}


Boolean SetAlarm(int num, int d,int p)
{
  Boolean b;

  b=false;
  if ((d==-2) && (p>0))
  {
    ToggleAlarm(num);
    b=true;
  } else if (prefs.world[num].options.active==0)
  {
    if (p==0)
    {
      if (d==-1)
      {
        historymode=num;
        FrmPopupForm(formID_history);
      } else if ((d==3) && (prefs.world[num].options.ampm==1))
        {
           ChangeAlarm(num,0,true,12);
           b=true;
        } else
      if (d>3)
      {
        ChangeDays(num,d-4);
        b=true;
      }
    } else
    {
      if (d==-1)
      {
        SetSound(num,p-1);
        b=true;
      } else
      {
        ChangeAlarm(num,d,p<3,12);
        b=true;
      }
    }
  }
  return b;
}

Boolean SetTimer(int num, int d,int p,int drawpage)
{
  Boolean b;
  long dummy;
  int val;
  int iii,jjj,kkk;


  b=false;

  if ((p==0) && (d!=-1))
  {
    iii=prefs.pages[prefs.pageset][drawpage];
    kkk= iii & 7;
    prefs.pages[prefs.pageset][drawpage]=(pnCTimer << 3)+kkk;
    b=true;
  }

  if (prefs.world[num].options.active==0)
  {
    if (p==0)
     {
      if (d==-1)
      {
        historymode=num;
        FrmPopupForm(formID_history);
      }
     } else
     {
       if ((d>=0)  && (d<=5))
       {
         if (!prefs.world[num].options.calc)
         {
           dummy=prefs.world[num].value / ((long)hoursInSeconds);
           if ( dummy >= 24)
             d-=2;
           //FrmCustomAlert(alertID_text,"x",NULL,NULL);

           ChangeAlarm(num,d,p<3,12);
         }
         else
           ChangeCalcTimer(num , d , p<3);

         prefs.world[num].reset=prefs.world[num].value;
         b=true;
       }
     }
   }
  return b;
}

void UpdateTimer(int num)
{
  BigClockAlarm alarm;
  DmOpenRef BCdb=NULL;

  BCdb=OpenAlarm();
  ReadAlarm(BCdb,num-5,&alarm);

  //is timermode with alarm?
  if (
       (prefs.world[num].options.mode>=3) &&
       (prefs.world[num].options.mode!=5) &&
       ((prefs.world[num].options.timerup==0) || (prefs.world[num].options.daterelative==1))
     )
     {
       alarm.days=128;
       alarm.sound=prefs.sounds[prefs.world[num].options.sound];
       alarm.options=prefs.world[num].options;
       alarm.options.ignoresystemsoundoff=prefs.ignoresystemsoundoff;
       alarm.snoozetime=prefs.world[num].snoozetime;
       alarm.snoozecount=prefs.world[num].snoozecount;
       alarm.snoozecurrent=0;

       StrCopy(alarm.name,prefs.world[num].name);

       if (prefs.world[num].options.daterelative==1)
       {
         TimSecondsToDateTime(prefs.world[num].date,&alarm.time);
       } else
       {
         //set alarm.time to date/time of the alarm

         // .last set to timgetseconds (=now) in controltap
         TimSecondsToDateTime(prefs.world[num].last,&alarm.time);
         // add timer value to now
         TimAdjust(&alarm.time,prefs.world[num].value);
       }
    } // active alarm
  else
     {
      alarm.options.active=false;
     }

  WriteAlarm(BCdb,num-5,&alarm);
  CloseAlarm(BCdb);

}



Boolean ControlTap(int num, int d,int p,int drawpage)
{
  BigClockAlarm alarm;
  DmOpenRef BCdb=NULL;
  RectangleType rc;
  int iii,kkk;

  timercommand=0;

  if (p>0)
  {
    if ((d==0) || (d==1))
    {
      if (prefs.world[num].options.active==1)
      {
        prefs.world[num].options.active=0;
      } else
      {
        prefs.world[num].options.active=1;
        prefs.world[num].last=TimGetSeconds();
      }
      if ((prefs.world[num].options.alternate==1) && (prefs.world[num].options.active==1))
      {
        //chess mode
        if (num==wTimer1) iii=wTimer2;
        else if (num==wTimer2) iii=wTimer1;
        else if (num==wTimer3) iii=wTimer4;
        else if (num==wTimer4) iii=wTimer3;
        prefs.world[iii].options.active=0; 
        UpdateTimer(iii);
      }


      UpdateTimer(num);
      SetNextAlarm(cardNo,appID);
    } else if (prefs.world[num].options.active==0)
    {
       if ((d==2) || (d==3))
       {
          if (p >=3 )
          {
            timercommand=2+10*num;
          }
          else
          {
            timercommand=1+10*num;
          }
       } else if (d==4)
       {
         if (p>=3)
           prefs.world[num].options.timerup=0;
         else
           prefs.world[num].options.timerup=1;
       } else if (d==5)
       {
         prefs.world[num].options.sound=p-1;
       }
    }
    //StrIToA(help, timercommand);
    //FrmCustomAlert(alertID_text,help,NULL,NULL);

    return true;
  } else
  {
    if ((p==0) && (drawpage>=0))
    {
      if (d ==-1)
      {
        historymode=num;
        FrmPopupForm(formID_history);
      }  else
      {
        iii=prefs.pages[prefs.pageset][drawpage];
        kkk= iii & 7;
        prefs.pages[prefs.pageset][drawpage]=(pnTimer << 3)+kkk;
      }
    } else
    return false;
  }

}


Boolean SetTime(int num, int d,int p)
{
  int digit;
  Boolean up;
  long off;
  char help[16];
  Boolean special;

  special=FrmGetActiveFormID() == formID_localtime;

  off=0;
  if (prefs.world[num].options.active==0)
  {

    if (p==0)
    {
    } else
    {
      up = p<3;
      if (d==0)
        off=(long)hoursInSeconds*12;
      else if (d==1)
        off=hoursInSeconds;
      else if (d==2)
        off=hoursInSeconds /6;
      else if (d==3)
        off=hoursInSeconds /60;
      else if ( ( (d==4) || (d==5) ) && (prefs.world[num].options.ampm!=0) && (prefs.world[num].options.seconds==0) && (!special))
        off=(long)hoursInSeconds*12;
      else if ( (d==4) && ((prefs.world[num].options.seconds!=0) || special))
        off=10;
      else if ( (d==5) && ((prefs.world[num].options.seconds!=0) || special))
        off=1;

      if (!up)
        off=-off;
    }
    prefs.world[num].value+=off;
    return true;
  } else
    return false;
}

Boolean ChangeMonthOffset(int num,int i)
{
  prefs.world[num].monthoffset+=i;
  return true;
}

Boolean SectionPenUp(int val)
{
  unsigned char pan;
  unsigned char num;

  pan=val >> 3;
  num= val & 7;


  if (timercommand>0)
  {
  //StrIToA(help, timercommand);
  // FrmCustomAlert(alertID_text,help,NULL,NULL);
  //StrIToA(help, (wTimer1+num);
  //FrmCustomAlert(alertID_text,help,NULL,NULL);

    if (timercommand==2+10*(wTimer1+num))
    {
      prefs.world[wTimer1+num].value=0;
    } else if (timercommand==(1+10*(wTimer1+num)))
    {
      prefs.world[wTimer1+num].value=prefs.world[wTimer1+num].reset;
      if (prefs.world[wTimer1+num].options.mode > 4)
      {
        prefs.world[wTimer1+num].options.timerup=0;
      }
    }
    //changed[0]=1;
    //changed[1]=1;
    SwitchDraw();
  } else
    return false;
}


Boolean SectionPenDown(int val, int x,int y,int drawpage)
{
  int d;
  int p;
  int ox;
  Boolean b;
  int dummy;
  unsigned char pan;
  unsigned char num;


  ox=x;
  if (val>=0)
  {
    pan=val >> 3;
    num= val & 7;
  }  else
  {
    pan=255;
    num=0;
  }


  // y parts
  if (y<17)
    p=0;
  else
    p=(y-17) / 10 +1;



  switch (pan)
  {
    case pnTime: x-=prefs.world[wLocal+num].add;
            break;
    case pnAlarm: x-=prefs.world[wAlarm1+num].add;
            break;
  }

  d=-3;
  if (p==0)
  {
    if (ox<80)
      d=-1;
    else if ((x>=82))
      d=3;
    else if ((x>=25) && (x<=80))
      d=4+(x-25) / 8;
  } else
  {
    if ((pan==pnTimer) && (prefs.world[wTimer1+num].options.calc)==1)
    {
      //calculated timer
      if ((x>=0) && (x<=23))
        d=5;
      else if ((x>=24) && (x<=47))
        d=4;
      else if ((x>=48) && (x<=71))
        d=3;
      else if ((x>=72) && (x<=95))
        d=2;
      else if ((x>=104) && (x<=127))
        d=1;
      else if ((x>=128) && (x<=151))
        d=0;
    } else
    {
      //normal time/timer/alarm display xx:yy:zz
      if ( (x<=-25))
        d=-2;
      else if ((x>=-24) && (x<=-1))
        d=-1;
      else if ((x>=0) && (x<=23))
        d=0;
      else if ((x>=24) && (x<=47))
        d=1;
      else if ((x>=56) && (x<=79))
        d=2;
      else if ((x>=80) && (x<=103))
        d=3;
      else if ((x>=112) && (x<=135))
        d=4;
      else if ((x>=136) && (x<=159))
        d=5;
    }
  }


  b=false;
  switch (pan)
  {
    case 255: b=SetTime(0, d,p);
             break;
    case pnTime: {
              b=SetTime(wLocal+num, d,p);
              break;
            }
    case pnAlarm: {
              b=SetAlarm(wAlarm1+num, d,p);
              break;
            }
    case pnTimer:  {
               b=SetTimer(wTimer1+num, d,p,drawpage);
               break;
             }
    case pnCTimer:  {
                  b=ControlTap(wTimer1+num,d,p,drawpage);
               break;
             }
    case pnControl: {
                     b=ControlTap(wTimer1+num,d,p,-1);
                 break;
              }
    case pnMonth:  {
               if (x<40)
                 b=ChangeMonthOffset(wLocal+num,-1);
               else if (x<80)
                 b=ChangeMonthOffset(wLocal+num,1);
               else
                 {
                   prefs.world[wLocal+num].monthoffset=0;
                   b=true;
                 }
               break;
             }
  }

  return b;
}


static Boolean bigclock(EventPtr event)
{
  FormPtr   form;
  FormPtr   sure;
  int       handled = 0;
  Word      w1;
  Word      w2;
  Boolean   b;

  CALLBACK_PROLOGUE

  switch (event->eType)
  {
  case frmOpenEvent:
    form = FrmGetActiveForm();
    FrmDrawForm(form);
    page=3;


    SetPageNames();
    CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+prefs.mode)),1);
    CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_pageset1+prefs.pageset)),1);

    UpdateTimes();
    SetPageResource(prefs.pageset,prefs.mode,Time[0].hour);
    SwitchDraw();

    DrawCaptionTime();
    handled = 1;
    break;
  case menuOpenEvent:
      if (iscolor)
      {
        MenuAddItem ( menuitemID_layout, menuitemID_color,'C', "Color...");
      }
    break;
  case frmUpdateEvent:
    form = FrmGetActiveForm();
    FrmDrawForm(form);
    SwitchDraw();
    handled=1;
    break;
  case menuEvent:
    switch (event->data.menu.itemID)
    {
    case menuitemID_about:
      FrmAlert(alertID_info);
      break;
    case menuitemID_help:
        FrmHelp(stringID_help);
      break;
    case menuitemID_layout:
        FrmPopupForm(formID_layout);
        handled=1;
      break;
    case menuitemID_localtime:
        FrmPopupForm(formID_localtime);
        handled=1;
      break;
    case menuitemID_world:
        FrmPopupForm(formID_world);
        handled=1;
      break;
    case menuitemID_global:
        FrmPopupForm(formID_global);
        handled=1;
      break;
    case menuitemID_alarm:
        FrmPopupForm(formID_alarm);
        handled=1;
      break;
    case menuitemID_timer:
        FrmPopupForm(formID_timer);
        handled=1;
      break;
    case menuitemID_sounds:
        obase=0;
        FrmPopupForm(formID_sound);
        handled=1;
      break;
    case menuitemID_color:
        FrmPopupForm(formID_color);
        handled=1;
      break;
   case menuitemID_reset:
        if (FrmAlert(formID_sure)==0)
          ResetData();
        handled=1;
      break;
    }
    handled = 1;
    break;

  case ctlSelectEvent:
   if ( (event->data.ctlEnter.controlID <= buttonID_m4) &&
        (event->data.ctlEnter.controlID >= buttonID_m1))
   {
     prefs.mode= event->data.ctlEnter.controlID - buttonID_m1;
     SetPageResource(prefs.pageset,prefs.mode,Time[0].hour);
     SwitchDraw();
     //handled = 1;
   } else if ( (event->data.ctlEnter.controlID <= buttonID_pageset2) &&
        (event->data.ctlEnter.controlID >= buttonID_pageset1))
   {
     prefs.pageset=event->data.ctlEnter.controlID-buttonID_pageset1;
     SetPageResource(prefs.pageset,prefs.mode,Time[0].hour);
     SwitchDraw();
     SetPageNames();
   }
   break;

  case keyDownEvent:
   form= FrmGetActiveForm ();

   if (prefs.hardwarebuttons)
   {
     if (event->data.keyDown.chr  == pageUpChr)
     {
       ControlTap(5, 0, 1,-1);
       SwitchDraw();
     } else if (event->data.keyDown.chr == pageDownChr)
     {
       if (prefs.world[5].options.active==1)
       {
         ControlTap(5, 0, 1,-1);
       }  else
       {
         timercommand=1+(10*wTimer1);
         SectionPenUp(16);
         timercommand=0;
       }
       SwitchDraw();
     }

   } else
   {
     CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+prefs.mode)),0);
     if (event->data.keyDown.chr  == pageUpChr)
       {
        if (prefs.mode<3)
          prefs.mode++;
        else prefs.mode=0;
        SwitchDraw();
        handled = 1;
       }
     else if (event->data.keyDown.chr == pageDownChr)
       {
        if (prefs.mode>0)
          prefs.mode--;
        else prefs.mode=3;
        SwitchDraw();
        handled = 1;
       }
      CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+prefs.mode)),1);
    }
    break;
  case penDownEvent:
    if ( (event->screenY >=ytop) && (event->screenY <=139) )
    {
      if (event->screenY >=ybottom)
          b=SectionPenDown(prefs.pages[prefs.pageset][prefs.mode*2+1],event->screenX,event->screenY-ybottom,prefs.mode*2+1);
      else
          b=SectionPenDown(prefs.pages[prefs.pageset][prefs.mode*2],event->screenX,event->screenY-ytop,prefs.mode*2);
      if (b)
        SwitchDraw();
    }
    break;
  case penUpEvent:
    if (event->screenY >=ybottom)
      b=SectionPenUp(prefs.pages[prefs.pageset][prefs.mode*2+1]);
    else
      b=SectionPenUp(prefs.pages[prefs.pageset][prefs.mode*2]);
    timercommand=0;
    if (b)
      SwitchDraw();

    break;

  case nilEvent:
    WinGetWindowExtent(&w1,&w2);
    if ((w1==160) && (w2==160))
      IdleDraw();

    handled = 1;
    break;
    default: break;
  }

  CALLBACK_EPILOGUE

  return handled;
}


void SetLayoutLabels()
{
  FormPtr   form;
  ListPtr    list;
  CharPtr    listItem;
  ControlPtr popup;
  int iii;
  unsigned char val;
  unsigned char pan;
  unsigned char num;

  form = FrmGetActiveForm();

  for (iii=0; iii<4 ; iii++)
    SetFieldText(editID_time+iii,prefs.pagenames[prefs.pageset][iii], 9);

  for (iii=0; iii<8; iii++)
  {
    val=prefs.pages[prefs.pageset][iii];
    pan=val >> 3;
    num=val & 7;

    list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_paneltypes));
    listItem = LstGetSelectionText(list, pan);
    if(!listItem) listItem = "?";
    popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_panel11g+iii));
    CategorySetTriggerLabel(popup, listItem);

    list=NULL;
    switch (pan)
    {
      case pnDate: case pnTime: case pnDateD: case pnMonth: case pnAnalog: /*case pnAnalogU: case pnAnalogL:*/{
          list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_timelist));
          break;
          }
      case pnAlarm: case pnTimer: case pnControl: case pnVTimer: case pnCTimer:{
          list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_alarmlist));
          break;
          }
      case pnWorld:  {
          list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_worldlist));
          break;
          }
      default: break;

    }
    if (list != NULL)
    {
     listItem = LstGetSelectionText(list, num);
      if(!listItem) listItem = "?";
      popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_panel11t+iii));
      CategorySetTriggerLabel(popup, listItem);
    }

    switch (pan)
    {
      case pnEmpty:  {
                 HideEdit(popupID_panel11t+iii);
                 HideEdit(checkboxID_time1a+iii);
                 HideEdit(checkboxID_time1b+iii);
                 break;
              }
      case pnTime: {
                 SetCheckBox(checkboxID_time1a+iii,prefs.world[wLocal+num].options.seconds==1);
                 ShowEdit(checkboxID_time1a+iii);
                 SetCheckBox(checkboxID_time1b+iii,prefs.world[wLocal+num].options.ampm!=1);
                 ShowEdit(checkboxID_time1b+iii);
                 ShowEdit(popupID_panel11t+iii);
                 break;
              }
      case pnAlarm:{
               HideEdit(checkboxID_time1a+iii);
               SetCheckBox(checkboxID_time1b+iii,prefs.world[wAlarm1+num].options.ampm != 1);
               ShowEdit(checkboxID_time1b+iii);
               ShowEdit(popupID_panel11t+iii);
               break;
             }
      case pnAnalog: {
              SetCheckBox(checkboxID_time1a+iii,prefs.world[wLocal+num].options.seconds==1);
              ShowEdit(checkboxID_time1a+iii);
              HideEdit(checkboxID_time1b+iii);
              ShowEdit(popupID_panel11t+iii);
              break;
            }
      default:  {
                   HideEdit(checkboxID_time1a+iii);
                   HideEdit(checkboxID_time1b+iii);
                   break;
                }
    }
  }
}

static Boolean layout(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       index;
  ListPtr   ListP;
  int iii;
  int num;
  unsigned char val;

  switch (event->eType)
  {
  case frmOpenEvent:
    form = FrmGetActiveForm();
    SetLayoutLabels();
    FrmDrawForm(form);
    page=1;
    handled = 1;
    break;


 case ctlSelectEvent:
   if (event->data.ctlEnter.controlID == buttonID_ok)
   {
     for (iii=0; iii<4 ; iii++)
       GetFieldText(editID_time+iii,prefs.pagenames[prefs.pageset][iii]);
     changed[0]=1;
     changed[1]=1;

     FrmReturnToForm(formID_bigclock);
     handled = 1;
   } else if (
               (event->data.ctlEnter.controlID >= popupID_panel11g) &&
               (event->data.ctlEnter.controlID <= popupID_panel42g)
             )
   {
     form = FrmGetActiveForm();
     index = FrmGetObjectIndex(form, popuplistID_paneltypes);
     ListP = FrmGetObjectPtr(form, index);
     num=event->data.ctlEnter.controlID-popupID_panel11g;
     LstSetPosition(ListP,0,30+(num/2)*34+(num % 2) * 10);

     val=(prefs.pages[prefs.pageset][num]);
     LstSetSelection(ListP,val >> 3);
     iii=LstPopupList(ListP);
     if (iii != -1)
     {
       val= (iii << 3);
       prefs.pages[prefs.pageset][num]=val;
     }
     SetLayoutLabels();
   } else if (
               (event->data.ctlEnter.controlID >= popupID_panel11t) &&
               (event->data.ctlEnter.controlID <= popupID_panel42t)
             )
   {
     form = FrmGetActiveForm();

     num=event->data.ctlEnter.controlID-popupID_panel11t;
     val=(prefs.pages[prefs.pageset][num]);

     switch (val >> 3)
     {
      case pnDate: case pnTime: case pnDateD: case pnMonth: case pnAnalog: {
          index=FrmGetObjectIndex(form, popuplistID_timelist);
          break;
          }
      case pnAlarm: case pnTimer: case pnControl: case pnVTimer: case pnCTimer:{
          index=FrmGetObjectIndex(form, popuplistID_alarmlist);
          break;
          }
      case pnWorld:  {
          index=FrmGetObjectIndex(form, popuplistID_worldlist);
          break;
          }
     }

     ListP = FrmGetObjectPtr(form, index);
     LstSetPosition(ListP,50,30+(num/2)*34+(num % 2) * 10);

     LstSetSelection(ListP,val & 7);

     iii=LstPopupList(ListP);
     if (iii != -1)
     {
       val= (val & 248) + iii;
       prefs.pages[prefs.pageset][num]=val;
     }
     SetLayoutLabels();
   } else if (
               (event->data.ctlEnter.controlID >= checkboxID_time1a) &&
               (event->data.ctlEnter.controlID <= checkboxID_timer2a)
             )
   {
       val=prefs.pages[prefs.pageset][event->data.ctlEnter.controlID-checkboxID_time1a];
       num=val >> 3;
       iii= val & 7;
       switch (num)
       {
         case pnTime: case pnAnalog: {
                   prefs.world[wLocal+iii].options.seconds=GetCheckBox(event->data.ctlEnter.controlID);
                   break;
                 }
       }
      SetLayoutLabels();
   } else if (
               (event->data.ctlEnter.controlID >= checkboxID_time1b) &&
               (event->data.ctlEnter.controlID <= checkboxID_timer2b)
             )
   {
       val=prefs.pages[prefs.pageset][event->data.ctlEnter.controlID-checkboxID_time1b];
       num=val >> 3;
       iii= val & 7;
       switch (num)
       {
         case pnTime:  {
                    prefs.world[wLocal+iii].options.ampm= ! GetCheckBox(event->data.ctlEnter.controlID);
                    break;
                  }
         case pnAlarm:  {
                   prefs.world[wAlarm1 + iii].options.ampm= ! GetCheckBox(event->data.ctlEnter.controlID);
                    break;
                 }
       }
      SetLayoutLabels();
   }
   break;

  case penDownEvent:
      break;
  default: break;
  }
  return handled;
}



void ResetData()
{
  BigClockAlarm alarm;
  int b;
  int iii;
  DmOpenRef BCdb=NULL;

  b=prefs.mode;

  BCdb=OpenAlarm();

  InitPref();
  for (iii=0 ; iii<aCount ; iii++)
  {
    ReadAlarm(BCdb,iii,&alarm);
    alarm.options.active=0;
    WriteAlarm(BCdb,iii,&alarm);
  }
  CloseAlarm(BCdb);
  SetNextAlarm(cardNo,appID);
  prefs.mode=b;
  SetPageNames();
  SwitchDraw();

}

void SetWorldOptions(int num)
{
  FormPtr   form;
  ListPtr    list;
  CharPtr    listItem;
  ControlPtr popup;
  Long h;
  Long m;
  char number[6];

  form= FrmGetActiveForm ();

  SetCheckBox(checkboxID_lock,prefs.world[num+wWorld1].options.active == 1);

  popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_tzd));
  list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,listID_tzd ));
  if (prefs.world[num+wWorld1].value < 0)
  {
    listItem = LstGetSelectionText(list, 1);
    LstSetSelection(list,1);
  } else
  {
    listItem = LstGetSelectionText(list, 0);
    LstSetSelection(list,0);
  }
  CtlSetLabel(popup,listItem);


  h=abs(prefs.world[num+wWorld1].value / (60*60));


  if (prefs.world[num+wWorld1].value < 0)
    m=abs((prefs.world[num+wWorld1].value + (h*60*60)) / 60);
  else
    m=abs((prefs.world[num+wWorld1].value - (h*60*60)) / 60);

  StrIToA(number,h);
  SetFieldText(editID_tzh,number,6);
  ShowEdit(editID_tzh);
  StrIToA(number,m);
  SetFieldText(editID_tzm,number,6);
  ShowEdit(editID_tzm);

  SetFieldText(editID_name,prefs.world[num+wWorld1].name,12);
  ShowEdit(editID_name);

}

void GetWorldEdit(int num)
{
  FormPtr   form;
  Long h;
  Long m;
  char text[8];
  ListPtr    list;

  form= FrmGetActiveForm ();

  GetFieldText(editID_tzh,text);
  h=StrAToI(text);


  GetFieldText(editID_tzm,text);
  m=abs(StrAToI(text));


  list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,listID_tzd ));
  if (LstGetSelection(list)==1)
    prefs.world[num+wWorld1].value=-(h*60*60+m*60);
  else
    prefs.world[num+wWorld1].value=h*60*60+m*60;

  GetFieldText(editID_name,prefs.world[num+wWorld1].name);
}

static Boolean optionsworld(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       index;
  ListPtr   ListP;
  BigClockAlarm alarm;
  int num;

  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    optionpage=0;
    SetWorldOptions(optionpage);
    CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+optionpage)),1);
    FrmDrawForm(form);
    handled = 1;
    page=2;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      GetWorldEdit(optionpage);
      changed[0]=1;
      changed[1]=1;

      FrmReturnToForm(formID_bigclock);
      handled = 1;
    } else if (
               (event->data.ctlEnter.controlID >= buttonID_ml) &&
               (event->data.ctlEnter.controlID <= buttonID_m4)
             )

    {
     GetWorldEdit(optionpage);
     optionpage=event->data.ctlEnter.controlID-buttonID_m1;
     SetWorldOptions(optionpage);
    } else if (event->data.ctlEnter.controlID == checkboxID_lock)
    {
      prefs.world[optionpage+wWorld1].options.active=GetCheckBox(checkboxID_lock);
    }
    break;
   default: break;

  }


  return handled;
}

static Boolean optionsglobal(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       index;

  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    SetCheckBox(checkboxID_hardware,prefs.hardwarebuttons);
    SetCheckBox(checkboxID_soundoff,prefs.ignoresystemsoundoff);

    FrmDrawForm(form);
    handled = 1;
    page=7;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    } else if (event->data.ctlEnter.controlID == checkboxID_hardware)
    {
       prefs.hardwarebuttons=GetCheckBox(checkboxID_hardware);
    } else if (event->data.ctlEnter.controlID == checkboxID_soundoff)
    {
       prefs.ignoresystemsoundoff=GetCheckBox(checkboxID_soundoff);
    }

    break;
   default: break;

  }


  return handled;
}



static Boolean optionslocaltime(EventPtr event)
{
  RectangleType rc = {{0,40+16},{160,40}};
  FormPtr   form;
  int       handled = 0;
  int       index;
  ListPtr   ListP;
  BigClockAlarm alarm;
  int num;
  ListPtr  list;
  Boolean b;
  Long now;
  Long now2;
  DateTimeType time1;
  DateTimeType time2;


  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:

    FrmDrawForm(form);

    if (iscolor)
      ColorSet(false);
    page=4;
    DrawTime(0,40,false,false,false);


    handled = 1;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      now=TimGetSeconds();
      TimSecondsToDateTime(now,&time1);
      TimSecondsToDateTime(now+prefs.world[0].value,&time2);
      time2.month=time1.month;
      time2.day=time1.day;
      time2.year=time1.year;
      now2=TimDateTimeToSeconds(&time2);
      TimSetSeconds(now2);

      if (GetCheckBox(checkboxID_keepworld) !=0)
      {
        prefs.world[1].value-=now2-now;;
        prefs.world[2].value-=now2-now;;
        prefs.world[3].value-=now2-now;;
        prefs.world[4].value-=now2-now;;
      }

      prefs.world[0].value=0;
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    } else if (event->data.ctlEnter.controlID == buttonID_cancel)
    {
      prefs.world[0].value=0;
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    }
    break;
  case penDownEvent:
    if ( (event->screenY >=24) && (event->screenY <=100) )
    {
      prefs.world[0].options.active=0;
      if (event->screenY >=60)
          b=SectionPenDown(-1,event->screenX,event->screenY-40,0);
      if (b)
      {
        UpdateTimes();
        WinEraseRectangle(&rc,0);
        DrawTime(0,40,false,false,false);
      }
      prefs.world[0].options.active=1;
      handled=1;
    }
    break;
  case nilEvent:
       if (UpdateTimes()>0)
       {
         WinEraseRectangle(&rc,0);
         DrawTime(0,40,false,false,false);
       }
       break;
    default: break;

  }
  return handled;
}


void SetAlarmOptions(int num)
{
  FormPtr   form;
  ListPtr    list;
  CharPtr    listItem;
  ControlPtr popup;
  int iii;
  char number[9];
  DateTimeType rel;

  form= FrmGetActiveForm ();

  {
    if (ao)
    {
      //alrm
      popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_base));
      CategorySetTriggerLabel(popup, prefs.world[wLocal+prefs.world[num+obase].options.base].name);

      list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_trigger));
      listItem = LstGetSelectionText(list, prefs.world[num+obase].options.trigger);
      popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_trigger));
      CategorySetTriggerLabel(popup, listItem);
    } else
    {
      //timer
      list = (ListPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form, popuplistID_mode));
      listItem = LstGetSelectionText(list, prefs.world[num+obase].options.mode);
      popup = (ControlPtr)FrmGetObjectPtr(form, FrmGetObjectIndex(form,popupID_mode));
      CategorySetTriggerLabel(popup, listItem);
      SetCheckBox(checkboxID_autooff,prefs.world[num+obase].options.autooff == 1);
      SetCheckBox(checkboxID_chess,prefs.world[num+obase].options.alternate == 1);
      SetCheckBox(checkboxID_calc,prefs.world[num+obase].options.calc == 1);
      StrIToA(number,prefs.world[num+obase].calc);
      SetFieldText(editID_calc,number,8);
      ShowEdit(editID_calc);


      if (romversion>=0x03103000)
      {
          TimSecondsToDateTime(prefs.world[num+obase].date,&rel);
          DateToAscii(rel.month,rel.day,rel.year,SysPref.dateFormat,reltimerdate);
          CtlSetLabel(FrmGetObjectPtr(form,FrmGetObjectIndex(form,selectorID_reldate )),reltimerdate);

          TimeToAscii(rel.hour,rel.minute,SysPref.timeFormat,reltimertime);
          CtlSetLabel(FrmGetObjectPtr(form,FrmGetObjectIndex(form,selectorID_reltime )),reltimertime);
          SetCheckBox(checkboxID_daterel,prefs.world[num+obase].options.daterelative == 1);
      } else
      {
        HideEdit(checkboxID_daterel);
        HideEdit(selectorID_reldate);
        HideEdit(selectorID_reltime);

      }

    }

    SetCheckBox(checkboxID_backlight,prefs.world[num+obase].options.backlight == 1);
    SetCheckBox(checkboxID_talelight,prefs.world[num+obase].options.talelight == 1);
    SetCheckBox(checkboxID_silent,prefs.world[num+obase].options.silent == 1);
    SetCheckBox(checkboxID_wait,prefs.world[num+obase].options.wait == 0);

    SetCheckBox(checkboxID_snooze,prefs.world[num+obase].options.snooze == 1);

    StrIToA(number,prefs.world[num+obase].snoozetime);
    SetFieldText(editID_snooze,number,6);
    ShowEdit(editID_snooze);

    StrIToA(number,prefs.world[num+obase].snoozecount);
    SetFieldText(editID_max,number,6);
    ShowEdit(editID_max);

  }
}


Boolean getalarmtext(int num)
{
  int number;
  char text[8];

  GetFieldText(editID_snooze,text);
  prefs.world[num+obase].snoozetime=StrAToI(text);
  GetFieldText(editID_max,text);
  prefs.world[num+obase].snoozecount=StrAToI(text);
  if (!ao)
  {
    GetFieldText(editID_calc,text);
    prefs.world[num+obase].calc=StrAToI(text);
  }

}

static Boolean optionsalarm(EventPtr event)
{
  FormPtr   form;
  int       handled  = 0;
  int       index;
  ListPtr   ListP;
  BigClockAlarm alarm;
  int num;
  DateTimeType now;
  BigClockOptions *bco;

  form = FrmGetActiveForm();
  if (event->eType == frmOpenEvent)
  {
    optionpage=0;
    SetAlarmOptions(optionpage);
    CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+optionpage)),1);
    FrmDrawForm(form);
    handled = 1;
    page=2;
  } else if (event->eType == ctlSelectEvent)
  {
     //AlarmOptionsSelectEvent(event);
   bco=&prefs.world[optionpage+obase].options;
   if (event->data.ctlEnter.controlID == buttonID_ok)
   {
     changed[0]=1;
     changed[1]=1;

     getalarmtext(optionpage);
     FrmReturnToForm(formID_bigclock);

     if (ao)
     {
       UpdateAlarm(wAlarm1);
       UpdateAlarm(wAlarm2);
       UpdateAlarm(wAlarm3);
       UpdateAlarm(wAlarm4);
     } else
     {
       UpdateTimer(wTimer1);
       UpdateTimer(wTimer2);
       UpdateTimer(wTimer3);
       UpdateTimer(wTimer4);
     }

     SetNextAlarm(cardNo,appID);

     handled = 1;
   } else if (event->data.ctlEnter.controlID == popupID_base)
   {
     index = FrmGetObjectIndex(form, popuplistID_times);
     ListP = FrmGetObjectPtr(form, index);
     LstSetDrawFunction(ListP,DrawWorldName);

     LstSetSelection(ListP,bco->base);
     LstSetPosition(ListP,60,20);
     num=LstPopupList(ListP);
     if (num != -1)
       bco->base=num;
     SetAlarmOptions(optionpage);
   } else if (
               (event->data.ctlEnter.controlID >= buttonID_m1) &&
               (event->data.ctlEnter.controlID <= buttonID_m4)
             )

   {
     getalarmtext(optionpage);
     optionpage=event->data.ctlEnter.controlID-buttonID_m1;
     SetAlarmOptions(optionpage);
   } else if (event->data.ctlEnter.controlID == checkboxID_backlight)
   {
     bco->backlight=GetCheckBox(checkboxID_backlight);
   } else if (event->data.ctlEnter.controlID == checkboxID_talelight)
   {
     bco->talelight=GetCheckBox(checkboxID_talelight);
   } else if (event->data.ctlEnter.controlID == checkboxID_silent)
   {
     bco->silent=GetCheckBox(checkboxID_silent);
   } else if (event->data.ctlEnter.controlID == checkboxID_wait)
   {
     bco->wait=inot[GetCheckBox(checkboxID_wait)];
   } else if (event->data.ctlEnter.controlID == checkboxID_snooze)
   {
     bco->snooze=GetCheckBox(checkboxID_snooze);
   } else if (event->data.ctlEnter.controlID == checkboxID_autooff)
   {
     bco->autooff=GetCheckBox(checkboxID_autooff);
   /*} else if (event->data.ctlEnter.controlID == checkboxID_hardware)
   {
     bco->hardwarebuttons=GetCheckBox(checkboxID_hardware);*/
   } else if (event->data.ctlEnter.controlID == checkboxID_daterel)
   {
     bco->daterelative=GetCheckBox(checkboxID_daterel);
   } else if (event->data.ctlEnter.controlID == checkboxID_chess)
   {
     bco->alternate=GetCheckBox(checkboxID_chess);
   } else if (event->data.ctlEnter.controlID == checkboxID_calc)
   {
     bco->calc=GetCheckBox(checkboxID_calc);
   } else if (event->data.ctlEnter.controlID == popupID_trigger)
   {
     index = FrmGetObjectIndex(form, popuplistID_trigger);
     ListP = FrmGetObjectPtr(form, index);

     LstSetSelection(ListP,bco->trigger);
     LstSetPosition(ListP,60,112);
     num=LstPopupList(ListP);
     if (num != -1)
       bco->trigger=num;
     SetAlarmOptions(optionpage);
   } else if (event->data.ctlEnter.controlID == popupID_mode)
   {
     index = FrmGetObjectIndex(form, popuplistID_mode);
     ListP = FrmGetObjectPtr(form, index);

     LstSetSelection(ListP,bco->mode);
     LstSetPosition(ListP,60,20);
     num=LstPopupList(ListP);
     if (num != -1)
       bco->mode=num;
     SetAlarmOptions(optionpage);
   } else if (event->data.ctlEnter.controlID == selectorID_reldate)
   {
      TimSecondsToDateTime(prefs.world[optionpage+obase].date,&now);
      now.second=0;
      if (SelectDay(selectDayByDay,&(now.month),&(now.day),&(now.year),"Select Timer Date"))
      {
         prefs.world[optionpage+obase].date=TimDateTimeToSeconds(&now);
         SetAlarmOptions(optionpage);
      }
   } else if (event->data.ctlEnter.controlID == selectorID_reltime)
   {
      TimSecondsToDateTime(prefs.world[optionpage+obase].date,&now);
      now.second=0;
      if (SelectOneTime(&(now.hour),&(now.minute),"Select Timer Time"))
      {
         prefs.world[optionpage+obase].date=TimDateTimeToSeconds(&now);
         SetAlarmOptions(optionpage);
      }
   }
  }
  return handled;
}

void setsoundlabels()
{
   int iii;
   char number[6];

   SetCheckBox(checkboxID_systemsound,prefs.sounds[obase].beep == 1);
   for (iii=0; iii<4; iii++)
   {
      StrIToA(number,prefs.sounds[obase].freq[iii]);
      SetFieldText(editID_soundf1+iii*4,number,5);
      ShowEdit(editID_soundf1+iii*4);

      StrIToA(number,prefs.sounds[obase].time[iii]);
      SetFieldText(editID_soundf1+iii*4+1,number,5);
      ShowEdit(editID_soundf1+iii*4+1);

      StrIToA(number,prefs.sounds[obase].amplitude[iii]);
      SetFieldText(editID_soundf1+iii*4+2,number,5);
      ShowEdit(editID_soundf1+iii*4+2);

      StrIToA(number,prefs.sounds[obase].pause[iii]);
      SetFieldText(editID_soundf1+iii*4+3,number,5);
      ShowEdit(editID_soundf1+iii*4+3);
   } 
  StrIToA(number,prefs.sounds[obase].count);
  SetFieldText(editID_soundr,number,5);
  ShowEdit(editID_soundr);
}

void getsoundlabels()
{
  char text[8];
  int number;
  int iii;

   for (iii=0; iii<4; iii++)
   {
      GetFieldText(editID_soundf1+iii*4,text);
      prefs.sounds[obase].freq[iii]=StrAToI(text);
      GetFieldText(editID_soundf1+iii*4+1,text);
      prefs.sounds[obase].time[iii]=StrAToI(text);
      GetFieldText(editID_soundf1+iii*4+2,text);
      prefs.sounds[obase].amplitude[iii]=StrAToI(text);
      GetFieldText(editID_soundf1+iii*4+3,text);
      prefs.sounds[obase].pause[iii]=StrAToI(text);
   }
  GetFieldText(editID_soundr,text);
  prefs.sounds[obase].count=StrAToI(text);
}

static testsound(int num)
{
  int iii;
  int jjj;
  Boolean quit;
  SndCommandType cmd;
  EventType	event;
  int wait=0;

  iii=0;
  jjj=0;
  quit=false;


  while ( (iii< prefs.sounds[num].count) && (!quit) )
  {
    EvtGetEvent(&event, wait);
    if (SysHandleEvent (&event)) continue;

    if (event.eType == penDownEvent) return;



    if (prefs.sounds[num].freq[jjj] >0)
    {
      cmd.cmd = sndCmdFreqDurationAmp;
      cmd.param1 = prefs.sounds[num].freq[jjj];
      cmd.param2 = prefs.sounds[num].time[jjj];
      cmd.param3 = prefs.sounds[num].amplitude[jjj];
      SndDoCmd(0, &cmd, 0);

    }
    wait=prefs.sounds[num].pause[jjj];
    jjj++;
    if (jjj>=4)
    {
      iii++;
      jjj=0;
    }

  }
}

static Boolean optionssound(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       index;
  ListPtr   ListP;
  int num;
  char soundname[12];
  //Word      midicount;
  //Handle    midih;

  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    obase=0;
    CtlSetValue(FrmGetObjectPtr(form,FrmGetObjectIndex(form,buttonID_m1+obase)),1);

    //no midi yet
    /*
    if (usemidi)
    {
      usemidi=SndCreateMidiList(NULL,true,&midicount,&midih);
      //StrIToA(help,midicount);
      if (midilist != NULL)
        MemPtrFree(midilist);
      //midilist=MemPtrNew(sizeof(* char) * midicount);
    }

    if (~ usemidi)
    {
      HideEdit(popupID_midi);
    }
    */

    setsoundlabels();
    FrmDrawForm(form);
    handled = 1;
    page=2;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      getsoundlabels();
      //if (midilist != NULL)
      //  MemPtrFree(midilist);
      FrmReturnToForm(formID_bigclock);
      handled = 1;
   } else if (event->data.ctlEnter.controlID == checkboxID_systemsound)
   {
     prefs.sounds[obase].beep=GetCheckBox(checkboxID_systemsound);
   } else if (event->data.ctlEnter.controlID == buttonID_test)
   {
     getsoundlabels();
     testsound(obase);
   } else if (
               (event->data.ctlEnter.controlID >= buttonID_m1) &&
               (event->data.ctlEnter.controlID <= buttonID_m4)
             )

   {
     getsoundlabels();
     obase=event->data.ctlEnter.controlID-buttonID_m1;
     setsoundlabels();
    }
    break;
    default: break;

   }
   return handled;
}

void drawcolorsquares(void)
{
    RectangleType rcb = {{40,32},{50,14}};
    RectangleType rcf = {{40,50},{50,14}};
    RectangleType rcbi = {{100,32},{50,14}};
    RectangleType rcfi = {{100,50},{50,14}};


    WinSetForeColor(prefs.colorb);
    WinDrawRectangle(&rcb,0);

    WinSetForeColor(prefs.colorf);
    WinDrawRectangle(&rcf,0);

    WinSetForeColor(prefs.colorbi);
    WinDrawRectangle(&rcbi,0);

    WinSetForeColor(prefs.colorfi);
    WinDrawRectangle(&rcfi,0);

    WinSetForeColor(UIColorGetTableEntryIndex(UIObjectFrame));
    WinDrawRectangleFrame(1,&rcb);
    WinDrawRectangleFrame(1,&rcf);
    WinDrawRectangleFrame(1,&rcbi);
    WinDrawRectangleFrame(1,&rcfi);


  }


void InitThemeList(UInt32 type,int listid,char *def)
{
  FormPtr Form;
  ListPtr  list;
  int     pos;
  int     num;
  DmSearchStateType state;
  UInt16  card;
  LocalID dbid;
  char    *start;
  ControlPtr popup;
  CharPtr    listItem;
  int d;


  Form= FrmGetActiveForm();
  list=FrmGetObjectPtr(Form,FrmGetObjectIndex(Form,listid));

  pos=0;
  num=0;
  ThemeList[0]=NULL;

  StrCopy(help,"<no theme>");
  ThemeList[num]=&help[pos];
  pos+=StrLen(&help[pos])+1;
  num++;
  d=0;

  while ( (num<maxthemes) &&
          (DmGetNextDatabaseByTypeCreator(num==1,&state,type,BCAppID,false,&card,&dbid) == 0)
        )
  {
    ThemeList[num]=&help[pos];
    DmDatabaseInfo(card,dbid,&help[pos],NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
    pos+=StrLen(&help[pos])+1;
    if (StrCompare(def,ThemeList[num])==0)
      d=num;
    num++;
  }

  LstSetListChoices(list,ThemeList,num);
  LstSetSelection(list,d);
}


static Boolean optionscolor(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       index;
  ListPtr   ListP;
  int num;
  IndexedColorType icolor;
  RGBColorType rgbcolor;
  LocalID id;


  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    page=5;

    if (ThemeDB != 0)
      CloseTheme();
    else
      FreeBigBitmaps();

    InitThemeList('BCth',listID_theme,prefs.theme);

    FrmDrawForm(form);
    drawcolorsquares();

    handled = 1;
    break;
  case penDownEvent:
      if ( (event->screenX >=40) && (event->screenX <=90) )
      {
        //left
        if ( (event->screenY >=32) && (event->screenY <=46) )
        {
          //top
          icolor=prefs.colorb;
          UIPickColor(&icolor,&rgbcolor,UIPickColorStartPalette,"Back","");
          prefs.colorb=icolor;
        } else if ( (event->screenY >=50) && (event->screenY <=64) )
        {
          //bottom
          icolor=prefs.colorf;
          UIPickColor(&icolor,&rgbcolor,UIPickColorStartPalette,"Text","");
          prefs.colorf=icolor;
        }
      } else if ( (event->screenX >=100) && (event->screenX <=1500) )
      {
        //right
        if ( (event->screenY >=32) && (event->screenY <=46) )
        {
          //top
          icolor=prefs.colorbi;
          UIPickColor(&icolor,&rgbcolor,UIPickColorStartPalette,"Back selected","");
          prefs.colorbi=icolor;
        } else if ( (event->screenY >=50) && (event->screenY <=64) )
        {
          //bottom
          icolor=prefs.colorfi;
          UIPickColor(&icolor,&rgbcolor,UIPickColorStartPalette,"Text selected","");
          prefs.colorfi=icolor;
        }
      }
    drawcolorsquares();
    //handled = 1;
    break;
  case lstSelectEvent:
       if (event->data.lstSelect.selection > 0)
         StrCopy(prefs.theme,ThemeList[event->data.lstSelect.selection]);
       else
         prefs.theme[0]=0;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      initbigbitmap();
      if (StrLen(prefs.theme) > 0)
      {
        OpenTheme(prefs.theme);
      }
      else
      {
        CreateBigBitmaps();
        changed[0]=1;
        changed[1]=1;
      }
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    } else
    if (event->data.ctlEnter.controlID == buttonID_info)
    {
      if (StrLen(prefs.theme) > 0)
      {
       id=DmFindDatabase(0,prefs.theme);
       ThemeDB=DmOpenDatabase(0,id,dmModeReadOnly);
       ThemeDefH=DmGetResource('tSTR',7169);
       if (ThemeDefH != 0)
       {
         ThemeDef=MemHandleLock(ThemeDefH);
         FrmCustomAlert(alertID_text,(char *)ThemeDef,NULL,NULL);
         MemHandleUnlock(ThemeDefH);
       }
       DmCloseDatabase(ThemeDB);
       ThemeDB=0;
      }
    } else
    if (event->data.ctlEnter.controlID == buttonID_delete)
    {
      if (StrLen(prefs.theme) > 0)
      {
        if (FrmCustomAlert(alertID_confirmation,prefs.theme,NULL,NULL) == 0)
        {
          id=DmFindDatabase(0,prefs.theme);
          DmDeleteDatabase(0,id);
          prefs.theme[0]=0;
          InitThemeList('BCth',listID_theme,prefs.theme);
          FrmDrawForm(form);
          drawcolorsquares();
        }
      }
    }
    break;

    default: break;

   }
   return handled;
}

void drawlabels()
{
  int iii;

  FntSetFont(2);
  for (iii=0; iii<8; iii++)
  {
    WinDrawChars(prefs.history[iii],StrLen(prefs.history[iii]),40,14+iii*14);
  }
}

void addtohistory(char *label)
{
  int iii;
  int free;
  Boolean found;

  found=false;
  iii=0;
  free=-1;

  while ((!found) && (iii<8))
  {
    if ((prefs.history[iii][0]==0) && (free==-1))
    {
      free=iii;
    } else if (StrCompare(label,prefs.history[iii])==0)
    {
      found=true;
    } else
      iii++;
  }
  if (!found)
  {
    if (free==-1)
    {
      MemMove(&prefs.history[0][0],&prefs.history[1][0],7*12);
      free=7;
    }
    StrCopy(prefs.history[free],label);
  }

}



static Boolean labelhistory(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       num;


  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    //setsoundlabels();
    page=6;

    FrmDrawForm(form);

    //FrmCustomAlert(alertID_text,"a",NULL,NULL);
    drawlabels();
    //FrmCustomAlert(alertID_text,"b",NULL,NULL);

    SetFieldText(editID_edit1, "",12);
    FrmSetFocus(form,FrmGetObjectIndex(form,editID_edit1));

    //FrmCustomAlert(alertID_text,"c",NULL,NULL);

    handled = 1;
    break;
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_cancel)
    {
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    } else if (event->data.ctlEnter.controlID == buttonID_set)
    {
      GetFieldText(editID_edit1,prefs.world[historymode].name);
      if (StrLen(prefs.world[historymode].name) > 0)
        addtohistory(prefs.world[historymode].name);

      changed[0]=1;
      changed[1]=1;
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    }
    break;
   case penDownEvent:
   {
     if ((event->screenY >= 14) && (event->screenY < 126))
     {
       num= (event->screenY - 14) / 14;
       StrCopy(prefs.world[historymode].name,prefs.history[num]);
       changed[0]=1;
       changed[1]=1;
       FrmReturnToForm(formID_bigclock);
       handled=1;
     }
     break;
    default: break;

   }

  }
  return handled;
}

static Boolean newyear(EventPtr event)
{
  FormPtr   form;
  int       handled = 0;
  int       iii;
  int       xa;
  int       xp;
  RectangleType rc = {{0,16},{160,144}};

  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case frmOpenEvent:
    FrmDrawForm(form);
    if (iscolor)
    {
      WinSetForeColor(255);
      WinSetBackColor(255);
      WinDrawRectangle(&rc,0);
    }


    page=-1;
    initrockets();
    newyeardone=1;

    addrocket(200,1600,5,-10,100,true,0);

    handled = 1;
    break;

  case penDownEvent:
    newyeardone=2;
    freerockets();
    FrmReturnToForm(formID_bigclock);
    handled = 1;
    break;
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == buttonID_ok)
    {
      newyeardone=2;
      freerockets();
      FrmReturnToForm(formID_bigclock);
      handled = 1;
    }
    break;
  case nilEvent:
    animaterockets();
    if (SysRandomUnder(120)==0)
    {
      xp=SysRandomUnder(1600);
      xa=SysRandomUnder(6);
      if (xp>800)
      {
        xa=-xa;
      }
      addrocket(xp,1600,xa,-10,SysRandomUnder(70)+50,true,SysRandomUnder(3));
      SndPlaySystemSound(sndWarning);
    }
    handled = 1;
    break;
    default: break;

   }
   return handled;
}




